@@ -26,7 +26,8 @@ from pytrainer.gui.dialogs import fileChooserDialog, guiFlush
 class garminTCXv2():
 	def __init__(self, parent = None, validate=False):
 		self.parent = parent
-		self.tmpdir = self.parent.conf.getValue("tmpdir")
+		self.pytrainer_main = parent.pytrainer_main
+		self.tmpdir = self.pytrainer_main.profile.tmpdir
 		self.data_path = os.path.dirname(__file__)
 		self.validate = validate
 		self.sport = self.getConfValue("Force_sport_to")
@@ -34,7 +35,7 @@ class garminTCXv2():
 	def getConfValue(self, confVar):
 		info = XMLParser(self.data_path+"/conf.xml")
 		code = info.getValue("pytrainer-plugin","plugincode")
-		plugindir = self.parent.conf.getValue("plugindir")
+		plugindir = self.pytrainer_main.profile.plugindir
 		if not os.path.isfile(plugindir+"/"+code+"/conf.xml"):
 			value = None
 		else:
@@ -58,9 +59,9 @@ class garminTCXv2():
 				for activity in activities:
 					if not self.inDatabase(activity):
 						sport = self.getSport(activity)
-						gpxfile = "%s/garmin-tcxv2-%d.gpx" % (self.tmpdir, len(importfiles))					
+						gpxfile = "%s/garmin-tcxv2-%d.gpx" % (self.tmpdir, len(importfiles))
 						self.createGPXfile(gpxfile, activity)
-						importfiles.append((gpxfile, sport))					
+						importfiles.append((gpxfile, sport))
 					else:
 						logging.debug("File:%s activity %d already in database. Skipping import." % (filename, activities.index(activity)) )
 			else:
@@ -74,7 +75,7 @@ class garminTCXv2():
 			logging.debug("Not validating %s" % (filename) )
 			return True
 		else:
-			xslfile = os.path.realpath(self.parent.parent.data_path)+ "/schemas/GarminTrainingCenterDatabase_v2.xsd"
+			xslfile = os.path.realpath(self.pytrainer_main.data_path)+ "/schemas/GarminTrainingCenterDatabase_v2.xsd"
 			from pytrainer.lib.xmlValidation import xmlValidator
 			validator = xmlValidator()
 			return validator.validateXSL(filename, xslfile)
@@ -89,7 +90,7 @@ class garminTCXv2():
 	def inDatabase(self, activity):
 		#comparing date and start time (sport may have been changed in DB after import)
 		time = self.detailsFromTCX(activity)
-		if self.parent.parent.ddbb.select("records","*","date_time_utc=\"%s\"" % (time)):
+		if self.pytrainer_main.ddbb.select("records","*","date_time_utc=\"%s\"" % (time)):
 			return True
 		else:
 			return False
@@ -120,5 +121,4 @@ class garminTCXv2():
 		#xml_doc = etree.parse(filename)
 		xml_doc = activity
 		result_tree = transform(xml_doc)
-		result_tree.write(gpxfile, xml_declaration=True)
-
+		result_tree.write(gpxfile, xml_declaration=True, encoding='UTF-8')
