@@ -19,9 +19,13 @@
 import os
 import re
 import logging
+import colorsys
+import math
+import traceback
 
 import pytrainer.lib.points as Points
 from pytrainer.lib.fileUtils import fileUtils
+from pytrainer.lib.uc import UC
 
 class Googlemaps:
     def __init__(self, data_path = None, waypoint = None, pytrainer_main=None):
@@ -30,14 +34,41 @@ class Googlemaps:
         self.waypoint=waypoint
         self.pytrainer_main = pytrainer_main
         self.htmlfile = "%s/googlemaps.html" % (self.pytrainer_main.profile.tmpdir)
+        self.uc = UC()
         logging.debug("<<")
+        
+    def colorLine(self, polyline, average, variance):
+        stdev = math.sqrt(variance)
+        for i in polyline:
+            speed = i[1]
+            speed = (speed - (average - 2*stdev))/(4*stdev)
+            speed = min(max(speed,0), 1)
+            rgb_tuple = colorsys.hsv_to_rgb(0.66-(speed*0.66), 1, 0.8)
+            rgb_tuple = (rgb_tuple[0] * 255,rgb_tuple[1] * 255,rgb_tuple[2] * 255)
+            i[2] = '#%02x%02x%02x' % rgb_tuple
+            
+    def colorLineAbs(self, polyline):
+        for i in polyline:
+            speed = i[1]
+            if 0 <= speed < 7.5: #walk
+                rgb_tuple = colorsys.hsv_to_rgb(0.66, 1, (speed/7.5)*0.6)
+            elif 7.5 <= speed < 15: #jog-run
+                speed = ((speed-7.5)/7.5)
+                rgb_tuple = colorsys.hsv_to_rgb(0.66-speed*0.66, 1, 0.6+speed*0.2)
+            elif 15 <= speed < 40: #cycle
+                speed = ((speed-15)/25.0)
+                rgb_tuple = colorsys.hsv_to_rgb(1-speed, 1, 0.8+speed*0.2)
+            else: # fast cycle
+                rgb_tuple = colorsys.hsv_to_rgb(0, 1, 1)
+            
+            rgb_tuple = (rgb_tuple[0] * 255,rgb_tuple[1] * 255,rgb_tuple[2] * 255)
+            i[2] = '#%02x%02x%02x' % rgb_tuple
+                
 
-    def drawMap(self,activity):
+    def drawMap(self,activity, linetype):
         '''Draw google map
             create html file using Google API version3
-            render using embedded Mozilla
-
-            info at http://www.pygtk.org/pygtkmozembed/class-gtkmozembed.html
+            render using embedded Webkit
         '''
         logging.debug(">>")
         points = []
@@ -45,19 +76,51 @@ class Googlemaps:
         pointlist = []
         polyline = []
 
-        list_values = activity.tracks
+        list_values = activity.tracks 
+        # (accum distance, elevation, total duration, speed, lat, lon, bpm, cadence, corrected elevation)
+        # (19.963867183986643, 7.34716797, 6488, 13.791899119294959, 43.53392358, -5.634736, 146, None, None)
         if list_values is not None and list_values != [] and len(list_values) > 0:
             minlat, minlon = float(list_values[0][4]),float(list_values[0][5])
             maxlat=minlat
             maxlon=minlon
+            
+            av_sum = 0
+            variance_sum = 0
+            n = 0
+            #logging.debug("\n, ".join(map(str, list_values)))
             for i in list_values:
+                if linetype==1:
+                    if i[3] is not None:
+                        val = i[3]
+                    else:
+                        val = 0
+                        logging.error("No valid speed value for trackpoint: distance: %s | lat: %s | lon: %s" %(i[0],i[4],i[5]))
+                elif linetype==2:
+                	val = i[6] if i[6] else 0
+                elif linetype==3:
+                	val = i[7] if i[7]!=None else 1
+                else:
+                    val = 1
+                    
+                variance_sum += (val)**2
+                av_sum += val
+                n += 1
+                
                 lat, lon = float(i[4]), float(i[5])
                 minlat = min(minlat, lat)
                 maxlat = max(maxlat, lat)
                 minlon = min(minlon, lon)
                 maxlon = max(maxlon, lon)
                 pointlist.append((lat,lon))
-                polyline.append("new google.maps.LatLng(%s, %s)" % (lat, lon))
+                polyline.append(["new google.maps.LatLng(%s, %s)" % (lat, lon), val, ""])
+                
+            av_speed = av_sum / float(n)
+            variance = (variance_sum / float(n)) - av_speed**2
+            variance = max(variance, 16)
+
+            self.colorLine(polyline, av_speed, variance)
+            #self.colorLineAbs(polyline)
+            
             logging.debug("minlat: %s, maxlat: %s" % (minlat, maxlat))
             logging.debug("minlon: %s, maxlon: %s" % (minlon, maxlon))
             points,levels = Points.encodePoints(pointlist)
@@ -68,16 +131,16 @@ class Googlemaps:
             timeMin = (float(activity.time) / 3600.0 - timeHours) * 60
             time = "%d%s %02d%s" % (timeHours, _("h"), timeMin, _("min"))
             startinfo = "<div class='info_content'>%s: %s</div>" % (activity.sport_name, activity.title)
-            finishinfo = "<div class='info_content'>%s: %s<br>%s: %s%s</div>" % (_("Time"), time, _("Distance"), activity.distance, activity.distance_unit)
+            finishinfo = "<div class='info_content'>%s: %s<br>%s: %s%s</div>" % (_("Time"), time, _("Distance"), activity.distance, self.uc.unit_distance)
             startinfo = startinfo.encode('ascii', 'xmlcharrefreplace') #Encode for html
             finishinfo = finishinfo.encode('ascii', 'xmlcharrefreplace') #Encode for html
-            self.createHtml_api3(polyline, minlat, minlon, maxlat, maxlon, startinfo, finishinfo, laps)
+            self.createHtml_api3(polyline, minlat, minlon, maxlat, maxlon, startinfo, finishinfo, laps, linetype)
         else:
             self.createErrorHtml()
         return self.htmlfile
         logging.debug("<<")
 
-    def createHtml_api3(self,polyline, minlat, minlon, maxlat, maxlon, startinfo, finishinfo, laps):
+    def createHtml_api3(self,polyline, minlat, minlon, maxlat, maxlon, startinfo, finishinfo, laps, linetype):
         '''
         Generate a Google maps html file using the v3 api
             documentation at http://code.google.com/apis/maps/documentation/v3
@@ -98,9 +161,9 @@ class Googlemaps:
         <script type="text/javascript" src="http://maps.google.com/maps/api/js?sensor=false"></script>
         <script type="text/javascript">
           function initialize() {\n'''
-        content += "            var startlatlng = %s ;\n" % (polyline[0])
+        content += "            var startlatlng = %s ;\n" % (polyline[0][0])
         content += "            var centerlatlng = new google.maps.LatLng(%f, %f);\n" % ((minlat+maxlat)/2., (minlon+maxlon)/2.)
-        content += "            var endlatlng = %s;\n" % (polyline[-1])
+        content += "            var endlatlng = %s;\n" % (polyline[-1][0])
         content += "            var swlatlng = new google.maps.LatLng(%f, %f);\n" % (minlat,minlon)
         content += "            var nelatlng = new google.maps.LatLng(%f, %f);\n" % (maxlat,maxlon)
         content += "            var startcontent = \"%s\";\n" % (startinfo)
@@ -198,19 +261,43 @@ class Googlemaps:
         content += '''
 
             var boundsBox = new google.maps.LatLngBounds(swlatlng, nelatlng );\n
-            map.fitBounds(boundsBox);\n
-            var polylineCoordinates = [\n'''
+            map.fitBounds(boundsBox);\n'''
+            
+        pre = 0
         for point in polyline:
-            content += "                                       %s,\n" % (point)
-        content += '''            ];\n
-            // Add a polyline.\n
-            var polyline = new google.maps.Polyline({\n
-                    path: polylineCoordinates,\n
-                    strokeColor: \"#3333cc\",\n
-                    strokeOpacity: 0.6,\n
-                    strokeWeight: 5,\n
-                    });\n
-            polyline.setMap(map);\n
+            if pre:
+                content += '''var polylineCoordinates = [\n'''
+                content += "                                       %s,\n" % (pre[0])
+                content += "                                       %s,\n" % (point[0])
+                content += '''            ];\n
+                    // Add a polyline.\n
+                    var polyline = new google.maps.Polyline({\n
+                            path: polylineCoordinates,\n
+                            strokeColor: \"%s\",\n
+                            strokeOpacity: 0.9,\n
+                            strokeWeight: 5,\n
+                            });\n
+                polyline.setMap(map);\n''' % point[2]
+                
+                contenttemplate = [
+                	"%s",
+                	"Speed: %0.1f km/h",
+                	"HR: %d bpm",
+                	"Cadence: %d",
+                ]
+                
+                content += '''
+                    google.maps.event.addListener(polyline, 'click', function(event) {
+                        var marker = new google.maps.InfoWindow({
+                          position: event.latLng, 
+                          content: "%s"
+                        });
+                        marker.setMap(map);
+                    });
+                    ''' % contenttemplate[linetype] % point[1]
+            pre = point
+        
+        content += '''
           }
 
         </script>
