@@ -16,26 +16,24 @@
 #along with this program; if not, write to the Free Software
 #Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-import gtkmozembed
-import os
-import re
-
 from pytrainer.extension import Extension
 from pytrainer.lib.fileUtils import fileUtils
-
-import string,cgi,time
-import time
-
+import gtk
 import logging
+import os
+import re
+import webkit
 
 class WaypointEditor:
 	def __init__(self, data_path = None, vbox = None, waypoint=None, parent=None):		
 		logging.debug(">>")
 		self.data_path = data_path
 		self.extension = Extension()
-		self.moz = gtkmozembed.MozEmbed()
-		self.moz.connect('title', self.handle_title_changed) 
-		vbox.pack_start(self.moz, True, True)
+		self.wkview = webkit.WebView()
+		self.wkview.connect('notify::title', self.handle_title_changed) 
+		scrolled_window = gtk.ScrolledWindow()
+		scrolled_window.add(self.wkview)
+		vbox.pack_start(scrolled_window, True, True)
 		vbox.show_all()
 		self.htmlfile = ""
 		self.waypoint=waypoint
@@ -43,7 +41,9 @@ class WaypointEditor:
 		logging.debug("<<")
 		
 	def handle_title_changed(self, *args): 
-		title = self.moz.get_title() 
+		title = self.wkview.get_title()
+		if title == None:
+			return
 		logging.debug("Received title: "+ title)
 		m = re.match("call:([a-zA-Z]*)[(](.*)[)]", title) 
 		if m: 
@@ -86,7 +86,7 @@ class WaypointEditor:
 		tmpdir = self.pytrainer_main.profile.tmpdir
 		htmlfile = tmpdir+"/waypointeditor.html"
 		logging.debug("HTML file: "+str(htmlfile))
-		self.moz.load_url("file://"+htmlfile)
+		self.wkview.load_uri("file://"+htmlfile)
 		logging.debug("<<")
 	
 	def createHtml(self,default_waypoint=None):
@@ -106,7 +106,7 @@ class WaypointEditor:
     <meta http-equiv="content-type" content="text/html; charset=utf-8"/>
     <title>edit waypoints</title>
 
-    <script id="googleapiimport" src="http://maps.google.com/maps?file=api&amp;v=2"
+    <script id="googleapiimport" src="http://maps.google.com/maps/api/js?sensor=false"
             type="text/javascript"></script>
     <script type="text/javascript">
 """
