@@ -17,7 +17,7 @@
 #Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 import matplotlib
-matplotlib.use('GTK')
+matplotlib.use('GTKAgg')
 from matplotlib.figure import Figure
 from matplotlib.backends.backend_gtkagg import FigureCanvasGTKAgg as FigureCanvasGTK
 from matplotlib.backends.backend_gtkagg import NavigationToolbar2GTKAgg as NavigationToolbar
@@ -171,7 +171,7 @@ class DrawArea:
         else:
             return '%1.1f' % x
 
-    def drawStackedBars(self,xvalues,yvalues,ylabel,title, valuesAreTime=False):
+    def drawStackedBars(self,xvalues,yvalues,ylabel,title, valuesAreTime=False, colors={}):
         '''function to draw stacked bars
             xvalues needs to be a list of lists of strings, e.g. [0]["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"]
             yvalues needs to be a list of dicts e.g. [0]{'Kayak': {'Tue': 10.08, 'Fri': 17.579999999999998, 'Thu': 15.66, 'Sat': 30.619999999999997}, {'Run': {'Mon': 9.65, 'Sun': 15.59}}
@@ -210,14 +210,20 @@ class DrawArea:
         xvals = [x+barOffset for x in range(0, numCols)]
         cellText = []
         self.showGraph=False
+        
+        for k in colors:
+        	if colors[k]==None: colors[k]=''
 
         #Display first axis
+        xticks = []
         for key in keys:
             logging.debug("Day of the week: %s", str(key))
             for ind in inds:
                 ybottoms[ind] += yheights[ind]
                 yheights[ind] = 0 #Zero heights
-            color = self.getColor(keys.index(key))
+            color = "#"+colors.get(key, '')
+            if len(color)<2:
+                color = self.getColor(keys.index(key))
             for xvalue in xvalues[0]:
                 index = xvalues[0].index(xvalue)
                 if xvalue in yvalues[0][key]:
@@ -237,6 +243,8 @@ class DrawArea:
         if len(xvalues) == 1:
             plt.title(title[0])
             axis.legend(loc=0)
+            
+        axis.set_xlim(0,numCols)
 
         logging.debug("X values first axis: %s", str(xvals))
         logging.debug("Y values first axis: %s", str(yheights))
@@ -252,7 +260,9 @@ class DrawArea:
                 for ind in inds:
                     ybottoms[ind] += yheights[ind]
                     yheights[ind] = 0.0 #Zero heights
-                color = self.getColor(keys.index(key))
+                color = "#"+colors.get(key, '')
+                if len(color)<2:
+                    color = self.getColor(keys.index(key))
                 for xvalue in xvalues[0]:
                     index = xvalues[0].index(xvalue)
                     if xvalue in yvalues[1][key]:
@@ -276,6 +286,7 @@ class DrawArea:
                     ax2.bar(xvals, [0]*numCols, bottom=[0]*numCols, width=barWidth, color=color,  align='edge', label=key)
                     pass
             ax2.set_xticklabels('' * len(xvalues[1]))
+            ax2.set_xlim(0,numCols)
             ax2.set_ylabel(ylabel[1])
             ax2.legend(loc=0)
             plt.title("%s vs %s" %(title[0],title[1]))
