@@ -16,18 +16,15 @@
 #along with this program; if not, write to the Free Software
 #Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-from SimpleGladeApp import SimpleGladeApp
+from SimpleGladeApp import SimpleBuilderApp
 import gtk
 import gobject
 import os
 
-class WindowExtensions(SimpleGladeApp):
+class WindowExtensions(SimpleBuilderApp):
 	def __init__(self, data_path = None, parent=None):
-		glade_path="glade/extensions.glade"
-		root = "extensions"
-		domain = None
 		self.parent = parent
-		SimpleGladeApp.__init__(self, data_path+glade_path, root, domain)
+		SimpleBuilderApp.__init__(self, "extensions.ui")
 
 	def new(self):
 		column_names=["id","name"]
