@@ -17,26 +17,30 @@
 #Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 from __future__ import division
-from SimpleGladeApp import SimpleGladeApp
+from SimpleGladeApp import SimpleBuilderApp
 from windowcalendar import WindowCalendar
-from pytrainer.equipment import EquipmentService
+from pytrainer.core.equipment import EquipmentService
 from pytrainer.gui.equipment import EquipmentUi
+from pytrainer.core.sport import Sport
 import gtk
 import gobject
 import logging
+import pytrainer
+import pytrainer.util.color
+from pytrainer.gui.color import ColorConverter
 
-class WindowProfile(SimpleGladeApp):
-    def __init__(self, data_path = None, parent=None, pytrainer_main=None):
-        glade_path="glade/profile.glade"
-        root = "newprofile"
-        domain = None
+class WindowProfile(SimpleBuilderApp):
+    def __init__(self, sport_service, data_path = None, parent=None, pytrainer_main=None):
         self.parent = parent
         self.pytrainer_main = pytrainer_main
         self.data_path = data_path
-        SimpleGladeApp.__init__(self, data_path+glade_path, root, domain)
+        SimpleBuilderApp.__init__(self, "profile.ui")
         self.conf_options = parent.profile_options
+        self.stored_color = pytrainer.util.color.Color(0)
+        self._sport_service = sport_service
 
     def new(self):
+        logging.debug(">>")
         self.gender_options = {
             0:_("Male"),
             1:_("Female")
@@ -56,61 +60,94 @@ class WindowProfile(SimpleGladeApp):
             self.prf_ddbb.insert_text(i,self.ddbb_type[i])
 
         #preparamos la lista sports:
-        column_names=[_("Sport"),_("MET"),_("Extra Weight"), _("Maximum Pace")]
+        column_names=[_("Sport"),_("MET"),_("Extra Weight"), _("Maximum Pace"), _("Color")]
         for column_index, column_name in enumerate(column_names):
-            column = gtk.TreeViewColumn(column_name, gtk.CellRendererText(), text=column_index)
+            if column_index==4:
+                renderer = gtk.CellRendererPixbuf()
+            else:
+                renderer = gtk.CellRendererText()
+            column = gtk.TreeViewColumn(column_name, text=column_index)
+            column.pack_start(renderer, expand=False)
+            if column_index==4:
+                column.add_attribute(renderer, 'pixbuf', column_index)
+            else:
+                column.add_attribute(renderer, 'text', column_index)
             column.set_resizable(True)
             self.sportTreeView.append_column(column)
 
         #initialise equipment tab:
         equipment_service = EquipmentService(self.pytrainer_main.ddbb)
         equipment_ui = EquipmentUi(self.data_path + "/glade", equipment_service)
-        self.equipment_container.add(equipment_ui)            
+        self.equipment_container.add(equipment_ui) 
+        logging.debug("<<")           
         
-    def setValues(self,list_options):
+    def present(self):
+        self.newprofile.present()
+        
+    def setValues(self, list_options):
+        logging.debug(">>")
+        # Need to think if it does make sense to use pprint -> compatibility issues
+        #print list_options
         for i in self.conf_options.keys():
             if not list_options.has_key(i):
-                print 'no list options for: ' + i
+                logging.info('No list options for %s' %s)
                 continue
             if i == "default_viewer":
                 if list_options[i] == "1":
-                    logging.debug("Setting defult map viewer to OSM")
+                    logging.info("Setting default map viewer to OSM")
                     self.radiobuttonDefaultOSM.set_active(1)
                 else:
-                    logging.debug("Setting defult map viewer to Google")
+                    logging.info("Setting default map viewer to Google")
                     self.radiobuttonDefaultGMap.set_active(1)
+            elif i == "prf_startscreen":
+                if list_options[i] == "current_day":
+                    self.radioButtonStartScreenCurrentDay.set_active(1)
+                else:
+                    self.radioButtonStartScreenLastEntry.set_active(1)
+                logging.info("Setting start screen to display %s" % list_options[i])
             else:
                 try:
                     var = getattr(self,i)
                 except AttributeError as e:
                     continue                
                 if i == "prf_hrzones_karvonen" or i == "prf_us_system":
-                    if list_options[i]=="True":
+                    if list_options[i] == "True":
+                        logging.info("Setting %s to true" %i)
                         var.set_active(True)
-    
                 elif i == "prf_gender":
                     for j in self.gender_options:
-                        if self.gender_options[j]==list_options[i]:
+                        if self.gender_options[j] == list_options[i]:
+                            logging.info("Setting gender to %s" % self.gender_options[j])
                             var.set_active(j)
                 elif i == "prf_ddbb":
                     for j in self.ddbb_type:
-                        if self.ddbb_type[j]==list_options[i]:
+                        if self.ddbb_type[j] == list_options[i]:
+                            logging.info("Setting %s as database engine" % self.ddbb_type[j])
                             var.set_active(j)
                             if j==0:
                                 self._ddbb_value_deactive()
                             else:
                                 self._ddbb_value_active()
                 else:
+                    logging.info("Setting %s to %s" % (i,list_options[i]))
                     var.set_text(list_options[i])
+        logging.debug("<<")
     
     def saveOptions(self):
+        logging.debug(">>")
         list_options = {}
+        print self.conf_options
         for i in self.conf_options.keys():
             if i == "default_viewer":
                 if self.radiobuttonDefaultOSM.get_active():
                     list_options[i] = "1"
                 else:
                     list_options[i] = "0"
+            elif i == "prf_startscreen":
+                ss_selected = "current_day"
+                if self.radioButtonStartScreenLastEntry.get_active():
+                    ss_selected = "last_entry"
+                list_options[i] = ss_selected
             else:
                 try:
                     var = getattr(self,i)
@@ -125,7 +162,11 @@ class WindowProfile(SimpleGladeApp):
                     list_options[i] = var.get_active_text()
                 else:
                     list_options[i] = var.get_text()
+            logging.info("Saving %s as %s" % (i, list_options[i]))
+        logging.info("Updating profile...")
         self.parent.setProfile(list_options)
+        self.parent.saveProfile()
+        logging.debug("<<")
     
     def on_calendar_clicked(self,widget):
         calendardialog = WindowCalendar(self.data_path,self)
@@ -138,49 +179,27 @@ class WindowProfile(SimpleGladeApp):
         #print widget, pointer, frame
         if frame==2:
             self.saveOptions()
-            sport_list = self.parent.getSportList()
-            logging.debug("Got sport_list: %s" % str(sport_list) )
-            if sport_list == 0:
-                pass
-            elif sport_list == -1:
-                self.sportlistbutton.set_label("It is not possible connect to the server")
-            else:
-                store = gtk.ListStore(
-                            gobject.TYPE_STRING,
-                            gobject.TYPE_STRING,
-                            gobject.TYPE_STRING,
-                            gobject.TYPE_STRING,
-                            object)
-                for i in sport_list:
-                    try:
-                        met = float(i[1])
-                    except:
-                        met = ""
-                    try:
-                        weight = float(i[2])
-                    except:
-                        weight = ""
-                    try:
-                        max_pace = int(i[4])
-                        if max_pace is None or max_pace == 0:
-                            max_pace = ""
-                    except Exception as e:
-                        #print type(e), e
-                        max_pace = ""
-                    
-                    
-                    iter = store.append()
-                    store.set (
-                                iter,
-                                0, str(i[0]),
-                                1, met,
-                                2, weight,
-                                3, max_pace,
-                                )
-                self.sportTreeView.set_model(store)
-                self.sportTreeView.set_cursor(0)
-                #self.sportlistbutton.hide()
-                self.sportlist.show()
+            sport_list = self._sport_service.get_all_sports()
+            store = gtk.ListStore(
+                        gobject.TYPE_STRING,
+                        gobject.TYPE_STRING,
+                        gobject.TYPE_STRING,
+                        gobject.TYPE_STRING,
+                        gtk.gdk.Pixbuf,
+                        object)
+            for sport in sport_list:
+                iter = store.append()
+                colorPixBuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, False, 8, 25, 15)
+                colorPixBuf.fill(sport.color.rgba_val)
+                store.set(iter,
+                    0, sport.name,
+                    1, sport.met,
+                    2, sport.weight,
+                    3, sport.max_pace,
+                    4, colorPixBuf)
+            self.sportTreeView.set_model(store)
+            self.sportTreeView.set_cursor(0)
+            self.sportlist.show()
         elif frame == 5: #Startup Parameters page selected
             self.init_params_tab()
     
@@ -204,12 +223,6 @@ class WindowProfile(SimpleGladeApp):
         else:
             self.checkbuttonValidate.set_active(False)
             
-        #Show if database and config check requested 
-        if self.pytrainer_main.startup_options.check:
-            self.checkbuttonCheck.set_active(True)
-        else:
-            self.checkbuttonCheck.set_active(False)
-            
         #Show if new graph activated
         if self.pytrainer_main.startup_options.newgraph:
             self.checkbuttonNewGraph.set_active(True)
@@ -240,18 +253,6 @@ class WindowProfile(SimpleGladeApp):
             logging.debug("Validate deactivated")
             self.pytrainer_main.startup_options.validate = False
             
-    def on_checkbuttonCheck_toggled(self, widget):
-        if self.checkbuttonCheck.get_active():
-            logging.debug( "Check activated")
-            if self.pytrainer_main.startup_options.check is not True:
-                #Need to do sanitycheck
-                logging.debug("Need to do sanitycheck")
-                self.pytrainer_main.sanityCheck()
-            self.pytrainer_main.startup_options.check = True
-        else:
-            logging.debug("Check deactivated")
-            self.pytrainer_main.startup_options.check = False
-                
     def on_checkbuttonNewGraph_toggled(self, widget):
         if self.checkbuttonNewGraph.get_active():
             logging.debug("NewGraph activated")
@@ -260,14 +261,6 @@ class WindowProfile(SimpleGladeApp):
             logging.debug("NewGraph deactivated")
             self.pytrainer_main.startup_options.newgraph = False       
     
-    def on_sportlistbutton_clicked(self,widget):
-        sport_list = self.parent.getSportList()
-        if sport_list == 0:
-            self.parent.build_ddbb()
-            self.sportlistbutton.hide()
-            self.sportlist.show()
-        
-            
     def on_accept_clicked(self,widget):
         self.saveOptions()
         self.close_window()
@@ -313,12 +306,22 @@ class WindowProfile(SimpleGladeApp):
         self.addsport.show()
 
     def on_newsport_accept_clicked(self,widget):
-        sport = self.newsportentry.get_text()
-        met = self.newmetentry.get_text()
-        weight = self.newweightentry.get_text()
-        maxpace = self.newmaxpace.get_text()
-        self.parent.addNewSport(sport,met,weight,maxpace)
-        self.parent.actualize_mainsportlist()
+        sport = Sport()
+        sport.name = unicode(self.newsportentry.get_text())
+        sport.met = self._trim_to_null(self.newmetentry.get_text())
+        sport.weight = self.newweightentry.get_text()
+        sport.max_pace = self._trim_to_null(self.newmaxpace.get_text())
+        sport.color = self.stored_color
+        if sport.name.lower() in [s.name.lower() for s in self._sport_service.get_all_sports()]:
+            msg = "Sport '%s' already exists" % sport.name
+            logging.error(msg)
+            md = gtk.MessageDialog(None, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, _(msg))
+            md.set_title(_("Sport Creation Error"))
+            md.run()
+            md.destroy()
+            return
+        self._sport_service.store_sport(sport)
+        self.pytrainer_main.refreshMainSportList()
         self.on_switch_page(None,None,2)
         self.hidesportsteps()
         self.buttonbox.set_sensitive(1)
@@ -334,9 +337,10 @@ class WindowProfile(SimpleGladeApp):
             self.deletesport.show()
 
     def on_deletesport_clicked(self,widget):
-        sport = self.sportnamedel.get_text()
-        self.parent.delSport(sport)
-        self.parent.actualize_mainsportlist()
+        sport_name = self.sportnamedel.get_text()
+        sport = self._sport_service.get_sport_by_name(sport_name)
+        self._sport_service.remove_sport(sport)
+        self.pytrainer_main.refreshMainSportList()
         self.on_switch_page(None,None,2)
         self.hidesportsteps()
         self.buttonbox.set_sensitive(1)
@@ -346,38 +350,61 @@ class WindowProfile(SimpleGladeApp):
         selected,iter = self.sportTreeView.get_selection().get_selected()
         if iter:
             self.buttonbox.set_sensitive(0)
-            sport = selected.get_value(iter,0)
-            name,met,weight,maxpace = self.parent.getSportInfo(sport)
-            self.editsportentry.set_text(sport)
-            self.sportnameedit.set_text(sport)
-            if weight is not None:
-                self.editweightentry.set_text(str(weight))
-            else:
-                self.editweightentry.set_text("")
-            if met is not None:
-                self.editmetentry.set_text(str(met))
-            else:
-                self.editmetentry.set_text("")
-            if maxpace is not None:
-                self.editmaxpace.set_text(str(maxpace))
-            else:
-                self.editmaxpace.set_text("")
+            sport_desc = selected.get_value(iter,0)
+            sport = self._sport_service.get_sport_by_name(sport_desc)
+            self.editsportentry.set_text(sport.name)
+            self.sportnameedit.set_text(sport.name)
+            self.editweightentry.set_text(str(sport.weight))
+            met_str = "" if sport.met is None else str(sport.met)
+            self.editmetentry.set_text(met_str)
+            max_pace_str = "" if sport.max_pace is None else str(sport.max_pace)
+            self.editmaxpace.set_text(max_pace_str)
+            colorPixBuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, False, 8, 250, 20)
+            colorPixBuf.fill(sport.color.rgba_val)
+            self.editcolor.set_from_pixbuf(colorPixBuf)
             self.hidesportsteps()
             self.editsport.show()
+            
+    def on_editcolor_clicked(self, widget):
+        selected,iter = self.sportTreeView.get_selection().get_selected()
+        if iter:
+            sport_desc = selected.get_value(iter,0)
+            sport = self._sport_service.get_sport_by_name(sport_desc)
+            colorseldlg = gtk.ColorSelectionDialog("test")
+            colorseldlg.colorsel.set_has_palette(True)
+            colorseldlg.colorsel.set_current_color(ColorConverter().convert_to_gdk_color(sport.color))
+            colorseldlg.run()
+            gdk_color = colorseldlg.colorsel.get_current_color()
+            self.stored_color = ColorConverter().convert_to_color(gdk_color)
+            colorPixBuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, False, 8, 250, 20)
+            colorPixBuf.fill(self.stored_color.rgba_val)
+            self.newcolor.set_from_pixbuf(colorPixBuf)
+            self.editcolor.set_from_pixbuf(colorPixBuf)
+            
+            colorseldlg.hide()
+            
+    def on_sporttreeview_row_activated(self, widget, path, column):
+        self.on_editsport_clicked(None)
     
     def on_editsport_accept_clicked(self,widget):
         oldnamesport = self.sportnameedit.get_text()
-        newnamesport = self.editsportentry.get_text()
-        newmetsport = self.editmetentry.get_text()
-        newweightsport = self.editweightentry.get_text()
-        newmaxpace = self.editmaxpace.get_text()
-        self.parent.updateSport(oldnamesport,newnamesport,newmetsport,newweightsport, newmaxpace)
-        self.parent.actualize_mainsportlist()
+        sport = self._sport_service.get_sport_by_name(oldnamesport)
+        sport.name = unicode(self.editsportentry.get_text())
+        sport.weight = self.editweightentry.get_text()
+        sport.met = self._trim_to_null(self.editmetentry.get_text())
+        sport.max_pace = self._trim_to_null(self.editmaxpace.get_text())
+        sport.color = self.stored_color
+        self._sport_service.store_sport(sport)
+        self.pytrainer_main.refreshMainSportList()
         self.on_switch_page(None,None,2)
         self.hidesportsteps()
         self.buttonbox.set_sensitive(1)
         self.sportlist.show()
         
+    def _trim_to_null(self, string):
+        trimmed = string.strip()
+        return None if trimmed == "" else trimmed
+        
     def on_sportcancel_clicked(self,widget):
         self.hidesportsteps()
         self.buttonbox.set_sensitive(1)
