@@ -19,36 +19,30 @@
 from __future__ import division
 import os
 import logging
+import traceback
 import gtk, gobject
-from SimpleGladeApp import SimpleGladeApp
+from SimpleGladeApp import SimpleBuilderApp
 from windowcalendar import WindowCalendar
 
-from filechooser import FileChooser
-from pytrainer.lib.date import Date
-from pytrainer.lib.unitsconversor import *
-import pytrainer.record
+from pytrainer.lib.date import getLocalTZ, time2second
 import dateutil.parser
-from dateutil.tz import * # for tzutc()
+from dateutil.tz import tzutc
 from pytrainer.lib.uc import UC
 
-class WindowRecord(SimpleGladeApp):
+class WindowRecord(SimpleBuilderApp):
     def __init__(self, equipment_service, data_path = None, listSport = None, parent = None, date = None, title=None, distance=None, time=None, upositive=None, unegative=None, bpm=None, calories=None, comment=None, windowTitle=None, equipment=[]):
         logging.debug(">>")
         self.parent = parent
         self.pytrainer_main = parent.pytrainer_main
-        #self.us = self.pytrainer_main.profile.prf_us_system #DEPRECATED
         self.uc = UC()
         logging.debug("Using US system: "+ str(self.uc.us))
         self.data_path = data_path
-        glade_path="glade/newrecord.glade"
-        root = "newrecord"
-        domain = None
         self.mode = "newrecord"
         self.id_record = ""
         self.store = None
         self.active_row = None
         self.activity_data = [] 
-        SimpleGladeApp.__init__(self, data_path+glade_path, root, domain)
+        SimpleBuilderApp.__init__(self, "newrecord.ui")
         self.conf_options = [
             "rcd_date",
             "rcd_sport",
@@ -67,8 +61,8 @@ class WindowRecord(SimpleGladeApp):
             "rcd_maxvel",
             ]
         self.listSport = {}
-        for i in listSport:
-            self.listSport[i[3]] = i[0] #Create dictionary using SportID as key (may be non sequential if sports have been deleted)
+        for sport in listSport:
+            self.listSport[sport.id] = sport.name
         for i in self.listSport:    
             self.rcd_sport.insert_text(i,self.listSport[i])
         self.rcd_sport.set_active(0)
@@ -86,7 +80,7 @@ class WindowRecord(SimpleGladeApp):
         if time != None:
             self.setTime(time)
         if distance!=None and time!=None:
-            self.on_calcaverage_clicked(None)
+            self.on_calcavs_clicked(None)
         if upositive != None:
             self.rcd_upositive.set_text(self.uc.height(upositive))
         if unegative != None:
@@ -95,13 +89,13 @@ class WindowRecord(SimpleGladeApp):
             self.rcd_calories.set_text(calories)
             
         #populate labels with units        
-        self.label_rcd_distance.set_text('Distance (%s)' %self.uc.unit_distance)
-        self.label_rcd_maxvel.set_text('Max (%s)' %self.uc.unit_speed)
-        self.label_rcd_average.set_text('Average (%s)' %self.uc.unit_speed)
-        self.label_rcd_maxpace.set_text('Max (%s)' %self.uc.unit_pace)
-        self.label_rcd_pace.set_text('Pace (%s)' %self.uc.unit_pace)
-        self.label_rcd_upositive.set_text('Ascent (%s)' %self.uc.unit_height)
-        self.label_rcd_unegative.set_text('Descent (%s)' %self.uc.unit_height)        
+        self.label_rcd_distance.set_text(_('Distance') + ' (%s)' %self.uc.unit_distance)
+        self.label_rcd_maxvel.set_text(_('Max') + ' (%s)' %self.uc.unit_speed)
+        self.label_rcd_average.set_text(_('Average') + ' (%s)' %self.uc.unit_speed)
+        self.label_rcd_maxpace.set_text(_('Max') + ' (%s)' %self.uc.unit_pace)
+        self.label_rcd_pace.set_text(_('Pace') + ' (%s)' %self.uc.unit_pace)
+        self.label_rcd_upositive.set_text(_('Ascent') + ' (%s)' %self.uc.unit_height)
+        self.label_rcd_unegative.set_text(_('Descent') + ' (%s)' %self.uc.unit_height)        
 
         self._init_equipment(equipment, equipment_service)
         logging.debug("<<")
@@ -196,7 +190,7 @@ class WindowRecord(SimpleGladeApp):
         while gtk.events_pending(): # This allows the GUI to update 
             gtk.main_iteration()    # before completion of this entire action
         #Select first row and display details
-        self.treeviewEntries.set_cursor(0)
+        self.treeviewEntries.set_cursor(0)  
         self.show_treeviewEntries_row(0)
         logging.debug("<<")
         
@@ -248,7 +242,8 @@ class WindowRecord(SimpleGladeApp):
                 activity["rcd_title"] = activity["rcd_title"].replace("\"","'")
                 #Add activity to DB etc
                 laps = activity.pop("laps", ())
-                self.activity_data[index]["db_id"] = self.parent.insertRecord(activity, laps)
+                selected_equipment_ids = self._get_selected_equipment_ids()
+                self.activity_data[index]["db_id"] = self.parent.insertRecord(activity, laps, equipment=selected_equipment_ids)
                 row += 1
             logging.debug("Processed %d rows of activity data" % row)
         else:
@@ -266,8 +261,11 @@ class WindowRecord(SimpleGladeApp):
             list_options["rcd_upositive"] = self.uc.usr2sys_str('height', self.rcd_upositive.get_text())
             list_options["rcd_unegative"] = self.uc.usr2sys_str('height', self.rcd_unegative.get_text())
             list_options["rcd_maxbeats"] = self.rcd_maxbeats.get_text()
-            list_options["rcd_pace"] = self.uc.usr2sys_str('pace', self.rcd_pace.get_text())
-            list_options["rcd_maxpace"] = self.uc.usr2sys_str('pace', self.rcd_maxpace.get_text())
+            # 2011.11.05 - dgranda
+            # Pace is shown to user in mm:ss format
+            # But pace in database is stored in a mixed way: 4.30 for 4:30 (4.5 in 'decimal'). This needs to be changed!!
+            list_options["rcd_pace"] = self.uc.usr2sys_str('pace', self.rcd_pace.get_text().replace(":","."))
+            list_options["rcd_maxpace"] = self.uc.usr2sys_str('pace', self.rcd_maxpace.get_text().replace(":","."))
             list_options["rcd_maxvel"] = self.uc.usr2sys_str('speed', self.rcd_maxvel.get_text())
             list_options["rcd_time"] = [self.rcd_hour.get_value_as_int(),self.rcd_min.get_value_as_int(),self.rcd_second.get_value_as_int()]
             buffer = self.rcd_comments.get_buffer()
@@ -275,19 +273,27 @@ class WindowRecord(SimpleGladeApp):
             comment = buffer.get_text(start,end, True)
             list_options["rcd_comments"] = comment.replace("\"","'")
             selected_equipment_ids = self._get_selected_equipment_ids()
+            # Added to change start time, only activities without GPX+ source file - dgranda 2011/06/10
+            record_time = self.rcd_starttime.get_text()
+            record_date = self.rcd_date.get_text()
+            localtz = getLocalTZ()
+            date = dateutil.parser.parse(record_date+" "+record_time+" "+localtz)
+            local_date = str(date)
+            utc_date = date.astimezone(tzutc()).strftime("%Y-%m-%dT%H:%M:%SZ")
+            list_options["date_time_utc"] = utc_date
+            list_options["date_time_local"] = local_date
 
             if self.mode == "newrecord":
                 logging.debug('Track data: '+str(list_options))
                 if list_options["rcd_gpxfile"] != "":
-                    logging.info('Adding new activity based on GPX file')   
-                    trackSummary=(list_options["rcd_sport"],"","")
-                    self.parent.insertNewRecord(list_options["rcd_gpxfile"], trackSummary)
+                    logging.info('Adding new activity based on GPX file')
+                    self.parent.insertRecord(list_options, None, selected_equipment_ids)
                 else:
                     logging.info('Adding new activity based on provided data')
                     #Manual entry, calculate time info
                     record_time = self.rcd_starttime.get_text()
                     record_date = self.rcd_date.get_text()
-                    localtz = Date().getLocalTZ()
+                    localtz = getLocalTZ()
                     date = dateutil.parser.parse(record_date+" "+record_time+" "+localtz)
                     local_date = str(date)
                     utc_date = date.astimezone(tzutc()).strftime("%Y-%m-%dT%H:%M:%SZ")
@@ -339,12 +345,12 @@ class WindowRecord(SimpleGladeApp):
             logging.debug("activity is None")
             logging.debug("<<")
             return
-        self.id_record = activity.id_record
+        self.id_record = activity.id
         (h, m, s) = activity.time_tuple
         self.rcd_hour.set_value(h)
         self.rcd_min.set_value(m)
         self.rcd_second.set_value(s)
-        self.rcd_date.set_text(activity.date)
+        self.rcd_date.set_text(str(activity.date))
         
         #self.rcd_distance.set_text("%.2f"%activity.distance)
         #myset_text(self.rcd_distance, 'distance', activity.distance, us=self.us, round=2)
@@ -358,8 +364,8 @@ class WindowRecord(SimpleGladeApp):
         self.rcd_upositive.set_text("%.2f" %self.uc.height(activity.upositive))
         self.rcd_unegative.set_text("%.2f" %self.uc.height(activity.unegative))
         self.rcd_maxvel.set_text("%.2f" %self.uc.speed(activity.maxspeed))
-        self.rcd_maxpace.set_text("%s" %self.parent.pace_from_float(self.uc.pace(activity.maxpace)))
-        self.rcd_pace.set_text("%s" %self.parent.pace_from_float(self.uc.pace(activity.pace)))
+        self.rcd_maxpace.set_text("%s" %self.parent.pace_from_float(self.uc.pace(activity.maxpace),True)) # value coming from DB
+        self.rcd_pace.set_text("%s" %self.parent.pace_from_float(self.uc.pace(activity.pace),True)) # value coming from DB
         self.rcd_maxbeats.set_text("%s"%activity.maxbeats)
         self.rcd_title.set_text(activity.title)
         
@@ -374,6 +380,9 @@ class WindowRecord(SimpleGladeApp):
             self.rcd_gpxfile.set_text(activity.gpx_file)
             self.frameGeneral.set_sensitive(0)      #Currently record values not changed if a GPX file is present
             self.frameVelocity.set_sensitive(0)     #Greying out options to indicate this to user
+            self.framePace.set_sensitive(0)
+            self.frameElevation.set_sensitive(0)
+            self.frameBeats.set_sensitive(0)
         logging.debug("<<")
         
     def setValues(self,values):
@@ -433,15 +442,32 @@ class WindowRecord(SimpleGladeApp):
         return None
     
     def on_calctime_clicked(self,widget):
+        logging.debug(">>")
         try:
-            distance = self.rcd_distance.get_text()
-            average = self.rcd_average.get_text()
-            time_in_hour = float(distance)/float(average)
+            distance = float(self.rcd_distance.get_text()) # distance is mandatory!
+            # we need either pace or speed
+            try:
+                average = float(self.rcd_average.get_text())
+                time_in_hour = distance/average
+                logging.debug("Distance: %0.3f km (mi) | Speed: %0.2f -> Time: %.f hours  " %(distance,average,time_in_hour)) 
+                pace = self.parent.pace_from_float(60/average)
+                logging.debug("Setting pace: %s" %pace)
+                self.rcd_pace.set_text(pace)
+            except:
+                pace_dec = self.parent.pace_to_float(self.rcd_pace.get_text())
+                time_in_hour = pace_dec*distance/60.0
+                logging.debug("Distance: %0.3f km (mi) | Pace_dec: %0.2f -> Time: %.f hours" %(distance,pace_dec,time_in_hour))
+                speed = distance/time_in_hour
+                logging.debug("Setting average speed: %0.2f" %speed)
+                self.rcd_average.set_text("%0.2f" %speed)
             self.set_recordtime(time_in_hour)
         except:
+            logging.debug("Traceback: %s" % traceback.format_exc())
             pass
+        logging.debug("<<")
 
     def update_activity_data(self, row, gpx_file, sport):
+        logging.debug(">>")
         self.activity_data[row]["rcd_comments"] = ""
         gpx_summary, laps = self.parent.summaryFromGPX(gpx_file, (sport,""))
         local_time = gpx_summary['date_time_local']
@@ -451,6 +477,7 @@ class WindowRecord(SimpleGladeApp):
         self.activity_data[row]["rcd_starttime"] = start_time
         self.activity_data[row]["date_time_local"] = gpx_summary['date_time_local']
         self.activity_data[row]["date_time_utc"] = gpx_summary['date_time_utc']
+        self.activity_data[row]["rcd_time"] = gpx_summary["rcd_time"]
         self.activity_data[row]["rcd_distance"] = gpx_summary["rcd_distance"]
         self.activity_data[row]["rcd_average"] = gpx_summary["rcd_average"]
         self.activity_data[row]["rcd_calories"] = gpx_summary["rcd_calories"]
@@ -464,6 +491,7 @@ class WindowRecord(SimpleGladeApp):
         self.activity_data[row]["rcd_title"] = ""
         self.activity_data[row]["laps"] = laps
         self.activity_data[row]["complete"] = True
+        logging.debug("<<")
 
         
     def show_treeviewEntries_row(self, row):
@@ -471,6 +499,7 @@ class WindowRecord(SimpleGladeApp):
         Show details of treeview entry
         TODO need to maintain any changes and display those....
         '''
+        logging.debug(">>")
         self.active_row = row
         #Get details from stored data
         #set sport
@@ -482,7 +511,7 @@ class WindowRecord(SimpleGladeApp):
         gpx_file = self.activity_data[row]["rcd_gpxfile"]
         self.setValue("rcd_gpxfile", gpx_file, "%s")
         #set duration
-        time = Date().time2second(self.activity_data[row]["rcd_time"])      #TODO Fix to use timeinseconds!!
+        time = time2second(self.activity_data[row]["rcd_time"])      #TODO Fix to use timeinseconds!!
         self.setTime(time)                                                  #TODO Fix to use timeinseconds!!
         #Set distance
         self.setValue("rcd_distance",self.activity_data[row]["rcd_distance"], "%s")
@@ -512,7 +541,9 @@ class WindowRecord(SimpleGladeApp):
                 gtk.main_iteration()    # before completion of this entire action
             #Get some info from gpx file
             self.update_activity_data(row, gpx_file, sport)
-        self.setValue("rcd_distance",self.activity_data[row]["rcd_distance"], "%s")  
+        self.setValue("rcd_distance",self.activity_data[row]["rcd_distance"], "%s") 
+        time = time2second(self.activity_data[row]["rcd_time"])
+        self.setTime(time)  
         self.setValue("rcd_date", self.activity_data[row]["rcd_date"], "%s")
         self.setValue("rcd_starttime", self.activity_data[row]["rcd_starttime"], "%s")
         self.setValue("rcd_average",self.activity_data[row]["rcd_average"])
@@ -525,6 +556,7 @@ class WindowRecord(SimpleGladeApp):
         self.rcd_pace.set_text(self.activity_data[row]["rcd_pace"])
         self.setValue("rcd_maxbeats",self.activity_data[row]["rcd_maxbeats"], "%s")
         self.rcd_title.set_text(self.activity_data[row]["rcd_title"])
+        logging.debug("<<")
         
         
     def on_rcd_title_changed(self, widget):
@@ -599,42 +631,33 @@ class WindowRecord(SimpleGladeApp):
             while gtk.events_pending(): # This allows the GUI to update 
                 gtk.main_iteration()    # before completion of this entire action
             self.show_treeviewEntries_row(path[0])
-        
-    def on_calcaverage_clicked(self,widget):
-        try:
-            hour = self.rcd_hour.get_value_as_int()
-            min = self.rcd_min.get_value_as_int()
-            sec = self.rcd_second.get_value_as_int()
-            time = sec + (min*60) + (hour*3600)
-            time_in_hour = time/3600.0
-            distance = float(self.rcd_distance.get_text())
-            average = distance/time_in_hour
-            self.rcd_average.set_text("%0.2f" %average)
-        except:
-            pass
-    
-    def on_calcpace_clicked(self,widget):
+
+    def on_calcavs_clicked(self,widget):
+        logging.debug(">>")
         hour = self.rcd_hour.get_value_as_int()
         min = self.rcd_min.get_value_as_int()
         sec = self.rcd_second.get_value_as_int()
         time = sec + (min*60) + (hour*3600)
         if time<1:
+            logging.debug("Seems no time value (%s) has been entered, nothing to calculate." %time)
             return False
-        time_in_min = time/60.0
         distance = float(self.rcd_distance.get_text())
         if distance<1:
+            logging.debug("Seems no distance value (%s) has been entered, nothing to calculate." %distance)
             return False
-        #Calc Pace 
-        average = time_in_min/distance
-        #Tranform pace to mm.ss
-        min = int(average)
-        per_min = average - min
-        sec = float(per_min) * 60 / 100
-        dec_pace = min + sec
+        logging.debug("Time: %d seconds | Distance: %0.2f km (mi)" %(time,distance))
+        # Average speed        
+        average_speed = distance*3600.0/time
+        logging.debug("Average speed: %0.2f" %average_speed)
+        self.rcd_average.set_text("%0.2f" %average_speed)
+        # Average pace 
+        dec_pace = 60/average_speed
         #Transform pace to mm:ss
         pace = self.parent.pace_from_float(dec_pace)
+        logging.debug("Average pace: %s" %pace)
         self.rcd_pace.set_text(pace)
-    
+        logging.debug("<<")
+
     def on_calccalories_clicked(self,widget):
         sport = self.rcd_sport.get_active_text()
         hour = self.rcd_hour.get_value_as_int()
@@ -660,17 +683,33 @@ class WindowRecord(SimpleGladeApp):
             self.rcd_calories.set_text(str(calories))
 
     def on_calcdistance_clicked(self,widget):
+        logging.debug(">>")
         try:
             hour = self.rcd_hour.get_value_as_int()
             min = self.rcd_min.get_value_as_int()
             sec = self.rcd_second.get_value_as_int()
             time = sec + (min*60) + (hour*3600)
             time_in_hour = time/3600.0
-            average = float(self.rcd_average.get_text())
-            distance = average*time_in_hour
+            # we need either pace or speed
+            try:
+                average = float(self.rcd_average.get_text())
+                distance = average*time_in_hour
+                logging.debug("Time: %d seconds | Speed: %0.2f -> Distance: %0.3f km (mi)" %(time,average,distance)) 
+                pace = self.parent.pace_from_float(60/average)
+                logging.debug("Setting pace: %s" %pace)
+                self.rcd_pace.set_text(pace)
+            except:
+                pace_dec = self.parent.pace_to_float(self.rcd_pace.get_text())
+                distance = time/(60.0*pace_dec)
+                logging.debug("Time: %d seconds | Pace_dec: %0.2f -> Distance: %0.3f km (mi)" %(time,pace_dec,distance))
+                speed = distance/time_in_hour
+                logging.debug("Setting average speed: %0.2f" %speed)
+                self.rcd_average.set_text("%0.2f" %speed)
             self.set_distance(distance) 
         except:
+            logging.debug("Traceback: %s" % traceback.format_exc())
             pass
+        logging.debug("<<")
     
     def set_distance(self,distance):
         self.rcd_distance.set_text("%0.2f" %distance)
@@ -692,8 +731,6 @@ class WindowRecord(SimpleGladeApp):
 
     def on_selectfile_clicked(self,widget):
         logging.debug(">>")
-        #self.filechooser = FileChooser(self.data_path,self,"set_gpxfile","open")
-        #self.filechooser.run()
         from pytrainer.gui.dialogs import fileChooserDialog
         selectedFile = fileChooserDialog(title="Choose a Google Earth file (.kml) to import", multiple=False).getFiles()
         if selectedFile is not None:
@@ -702,8 +739,6 @@ class WindowRecord(SimpleGladeApp):
 
     def set_gpxfile(self):
         logging.debug(">>")
-        #namefile = self.filechooser.filename
-        #self.rcd_gpxfile.set_text(namefile)
         logging.debug("<<")
 
     def on_calculatevalues_clicked(self,widget):
