@@ -20,10 +20,7 @@
 #Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 import logging
-import traceback
-import commands, os
 import dateutil
-from pytrainer.lib.date import Date
 
 #Define the tables and their columns that should be in the database
 #Obviously, this is not a list but a dict -> TODO: ammend name to avoid confusion!!!
@@ -53,6 +50,7 @@ tablesList = {  "records":{     "id_record":"integer primary key autoincrement",
                                         "weight":"float",
                                         "met":"float",
                                         "max_pace":"integer",
+                                        "color":"char(6)",
                                         },
                         "waypoints":{   "id_waypoint":"integer primary key autoincrement",
                                         "lat":"float",
@@ -73,6 +71,12 @@ tablesList = {  "records":{     "id_record":"integer primary key autoincrement",
                                         "end_lat": "float",
                                         "end_lon": "float",
                                         "calories": "int",
+                                        "intensity": "varchar(7)",
+                                        "laptrigger": "varchar(9)",
+                                        "max_speed": "float",
+                                        "avg_hr": "int",
+                                        "max_hr": "int",
+                                        "comments":"text",
                                         },
                         "athletestats": {
                                         "id_athletestat": "integer primary key autoincrement",
@@ -96,12 +100,15 @@ tablesList = {  "records":{     "id_record":"integer primary key autoincrement",
                                      "equipment_id": "int",
                                      }
                         }
-tablesDefaultData = {  "sports": [({ "name":"Mountain Bike" } ), ( {"name": "Bike"}), ({"name": "Run"}) ]}
+tablesDefaultData = { "sports": [
+    ({ "name": u"Mountain Bike", "weight": 0.0, "color": "0000ff" } ),
+    ({ "name": u"Bike", "weight": 0.0, "color": "00ff00"}),
+    ({ "name": u"Run", "weight": 0.0, "color": "ffff00"})
+]}
 
 
 class DDBB:
-    def __init__(self, configuration, pytrainer_main=None):
-        self.pytrainer_main = pytrainer_main
+    def __init__(self, configuration):
         self.configuration = configuration
         self.ddbb_type = self.configuration.getValue("pytraining","prf_ddbb")
         if self.ddbb_type == "mysql": #TODO no longer supported?
@@ -109,35 +116,25 @@ class DDBB:
         else:
             from sqliteUtils import Sql
 
-        self.confdir = self.configuration.confdir
-        self.ddbb_path = "%s/pytrainer.ddbb" %self.confdir
-
         ddbb_host = self.configuration.getValue("pytraining","prf_ddbbhost")
         ddbb = self.configuration.getValue("pytraining","prf_ddbbname")
         ddbb_user = self.configuration.getValue("pytraining","prf_ddbbuser")
         ddbb_pass = self.configuration.getValue("pytraining","prf_ddbbpass")
         self.ddbbObject = Sql(ddbb_host,ddbb,ddbb_user,ddbb_pass,self.configuration)
+        
+    def get_connection_url(self):
+        return self.ddbbObject.get_connection_url()
 
     def connect(self):
         connection_ok, connection_msg = self.ddbbObject.connect()
         if not connection_ok:
-			print "ERROR: Unable to connect to database"
-			print connection_msg
-			sys.exit(connection_ok)
-        #Do a quick check to ensure all tables are present in DB
-        if not self.checkDBTables():
-            #Some tables missing - do DB check
-            self.checkDBIntegrity()
+            print "ERROR: Unable to connect to database"
+            print connection_msg
+            sys.exit(connection_ok)
 
     def disconnect(self):
         self.ddbbObject.disconnect()
 
-    def build_ddbb(self): #TODO Is this needed?
-        self.ddbbObject.createDDBB()
-        self.ddbbObject.connect()
-        self.ddbbObject.createTables()
-        self.ddbbObject.createTableVersion()
-
     def select(self,table,cells,condition=None, mod=None):
         return self.ddbbObject.select(table,cells,condition,mod)
 
@@ -218,6 +215,10 @@ class DDBB:
             #Text so is OK??
             #TODO check length against spec?
             return value
+        elif cell_type.startswith('char'):
+            #Text so is OK??
+            #TODO check length against spec?
+            return value
         elif cell_type.startswith('date'):
             try:
                 result = dateutil.parser.parse(value).date()
@@ -287,158 +288,16 @@ class DDBB:
         ret_val = self.ddbbObject.freeExec(sql)
         return ret_val[0][0]
         
-    def checkDBTables(self):
-        '''Quick check that all expected tables existing in DB
-            return True if OK, False if any tables are missing
-        '''
-        global tablesList
-        logging.debug('>>')
-        tablesDB = self.ddbbObject.getTableList()
-        logging.debug('Tables in DB: '+str(tablesDB));
-        #Check Tables
+    def create_tables(self, add_default=True):
+        """Initialise the database schema from an empty database."""
+        logging.info("Creating database tables")
         for entry in tablesList:
-            logging.debug("Entry: "+entry)
-            if entry not in tablesDB:
-                return False
-        return True
-
-    def checkDBIntegrity(self):
-        '''17.11.2009 - dgranda
-        Retrieves tables and columns from database, checks current ones and adds something if missed. New in version 1.7.0
-        args: none
-        returns: none'''
-        global tablesList
-        global tablesDefaultData
-        logging.debug('>>')
-        logging.info('Checking PyTrainer database')
-        #if self.ddbb_type != "sqlite":
-        #    logging.error('Support for MySQL database is decommissioned, please migrate to SQLite. Exiting check')
-        #    exit(-2)
-        try:
-            tablesDB = self.ddbbObject.getTableList()
-        except:
-            logging.error('Not able to retrieve which tables are in DB. Printing traceback')
-            traceback.print_exc()
-            exit(-1)
-
-        logging.debug('Found '+ str(len(tablesDB))+' tables in DB: '+ str(tablesDB))
-
-        # Create a compressed copy of current DB
-        try:
-            self.ddbbObject.createDatabaseBackup()
-        except:
-            logging.error('Not able to make a copy of current DB. Printing traceback and exiting')
-            traceback.print_exc()
-            exit(-1)
-
-        #Check Tables
-        for entry in tablesList:
-            if entry not in tablesDB:
-                logging.warn('Table '+str(entry)+' does not exist in DB')
-                self.ddbbObject.createTableDefault(entry,tablesList[entry])
-                #Check if this table has default data to add..
-                if entry in tablesDefaultData:
-                    logging.debug("Adding default data to %s" % entry)
-                    for data_dict in tablesDefaultData[entry]:
-                        self.insert_dict(entry, data_dict)
-            else:
-                self.ddbbObject.checkTable(entry,tablesList[entry])
-
-        #Run any functions to update or correct data
-        #These functions _must_ be safe to run at any time (i.e. not be version specfic or only safe to run once)
-        self.populate_date_time_local()
-        self.populate_duration_from_time()
-        logging.debug('<<')
-
-    def checkDBDataValues(self):
-        ''' Check all data in DB and report values that do not match the type '''
-        global tablesList
-
-        for table in tablesList.keys():
-            pass
-
-    def populate_duration_from_time(self):
-        '''
-        Populate duration from time field
-            only for time can be parsed as an int
-            
-            also check pace value
-        '''
-        logging.debug('--')
-        #listOfRecords = self.select_dict("records",('id_record','time'), "duration is NULL")
-        #logging.debug("Found %d records in DB without date_time_local field populated" % (len(listOfRecords) ) )
-        listOfRecords = self.select_dict("records",('id_record','time', 'duration', 'pace', 'average', 'maxspeed', 'maxpace'))
-        logging.debug("Found %d records in DB" % (len(listOfRecords) ) )
-        for record in listOfRecords:
-            data = {}
-            try:
-                duration = int(record['time'])
-            except Exception as e:
-                logging.info( "Error parsing time (%s) as int for record_id: %s" % (record['time'], record['id_record']))
-                continue
-            if duration != record['duration']:
-                logging.debug("setting record %s duration to %d" % (record['id_record'], duration))
-                data['duration'] = duration
-            #Check pace
-            try:
-                db_avg_speed = float(record['average']) #km/hr
-                db_pace = float(record['pace']) #min/km
-                db_max_speed = float(record['maxspeed'])
-                db_max_pace = float(record['maxpace'])
-                calc_pace = 60 / db_avg_speed
-                calc_max_pace = 60 / db_max_speed
-                #Update DB if db_pace != calc_pace 
-                if "%.2f" % db_pace != "%.2f" % calc_pace:
-                    print "DB Pace: %.2f, Calc: %.2f" % (db_pace, calc_pace)
-                    #data['pace'] = calc_pace
-                else:
-                    print "Pace ok: %.2f" % db_pace
-                #Update DB if db_max_pace != calc_max_pace 
-                if "%.2f" % db_max_pace != "%.2f" % calc_max_pace:
-                    print "DB Max Pace: %.2f, Calc: %.2f" % (db_max_pace, calc_max_pace)
-                    #data['max_pace'] = calc_max_pace
-                else:
-                    print "Max pace ok: %.2f" % db_max_pace
-            except Exception as e:
-                #print type(e), e
-                logging.debug("Error with converting pace (%s), average speed(%s), max speed (%s), max pace(%s)" % (record['pace'], record['average'], record['maxspeed'], record['maxpace']))
-            #Update DB if any changes
-            if len(data) > 0:
-                self.update_dict("records",data ,"id_record = %d"%record['id_record'])
-
-    def populate_date_time_local(self):
-        ''' Populate date_time_local and date from date_time_utc
-                only access records that date_time_local is NULL
-                using OS timezone to create local_time
-
-                also updates date if date != local_time
-
-                TODO - leaves date_time_local blank for records that have been manually created (as date_time_utc will actually be local time)
-        '''
-        logging.debug('--')
-        listOfRecords = self.ddbbObject.select("records","id_record,date,date_time_utc,date_time_local", "date_time_local is NULL")
-        logging.debug("Found %d records in DB without date_time_local field populated" % (len(listOfRecords) ) )
-        for record in listOfRecords:
-            try:
-                gpxfile = self.configuration.gpxdir+"/%s.gpx"%(record[0])
-                dateFromUTC = Date().getDateTime(record[2])
-                if os.path.isfile(gpxfile) : #GPX file exists for this record - probably not a manual record
-                    date_time_local = str(dateFromUTC[1])
-                    dateFromLocal = dateFromUTC[1].strftime("%Y-%m-%d")
-                    if record[1] != dateFromLocal:
-                        #date field incorrect - update it
-                        logging.debug("Updating record id: %s with date: %s and date_time_local: %s" % (record[0], dateFromLocal, date_time_local) )
-                        self.ddbbObject.update("records","date, date_time_local",[dateFromLocal, date_time_local], "id_record = %d" %record[0])
-                    else:
-                        #date field OK, just update date_time_local
-                        logging.debug("Updating record id: %s with date_time_local: %s" % (record[0], date_time_local) )
-                        self.ddbbObject.update("records","date_time_local",[date_time_local], "id_record = %d" %record[0])
-                else: #Manual entry?
-                    #For manual entries, the UTC time is the local time
-                    #TODO figure out a way to correct this...
-                    pass
-            except Exception as e:
-                print "Error updating record: " + str(record)
-                print e
-                logging.debug("Error updating record: " + str(record))
-                logging.debug(str(e))
+            self.ddbbObject.createTableDefault(entry, tablesList[entry])
+            if add_default and entry in tablesDefaultData:
+                logging.debug("Adding default data to %s" % entry)
+                for data_dict in tablesDefaultData[entry]:
+                    self.insert_dict(entry, data_dict)
+                
+    def create_backup(self):
+        """Create a backup of the current database."""
+        self.ddbbObject.createDatabaseBackup()
