@@ -1,4 +1,4 @@
-# -*- coding: iso-8859-1 -*-
+# -*- coding: utf-8 -*-
 
 #Copyright (C) Fiz Vazquez vud1@sindominio.net
 
@@ -26,7 +26,7 @@ import time
 from datetime import datetime
 import logging
 from lxml import etree
-from pytrainer.lib.date import Date
+from pytrainer.lib.date import getDateTime
 
 # use of namespaces is mandatory if defined
 mainNS = string.Template(".//{http://www.topografix.com/GPX/1/1}$tag")
@@ -47,8 +47,11 @@ endPointTag = gpxdataNS.substitute(tag="endPoint")
 startPointTag = gpxdataNS.substitute(tag="startPoint")
 elapsedTimeTag = gpxdataNS.substitute(tag="elapsedTime")
 distanceTag = gpxdataNS.substitute(tag="distance")
+intensityTag = gpxdataNS.substitute(tag="intensity")
+triggerTag = gpxdataNS.substitute(tag="trigger")
+summaryTag = gpxdataNS.substitute(tag="summary")
 
-pytrainerNS = string.Template(".//{http://sourceforge.net.project/pytrainer/GPX/0/1}$tag")
+pytrainerNS = string.Template(".//{http://sourceforge.net/projects/pytrainer/GPX/0/1}$tag")
 pyt_eleTag = pytrainerNS.substitute(tag="ele")
 
 class Gpx:
@@ -63,14 +66,16 @@ class Gpx:
         self.trkpoints = []
         self.vel_array = []
         self.total_dist = 0
+        self.total_dist_trkpts = 0
         self.total_time = 0
+        self.total_time_trkpts = 0
         self.upositive = 0
         self.unegative = 0
         self.maxvel = 0
         self.maxhr = 0
         self.hr_average = 0
         self.date = ""
-        #self.Date = Date()
+        self.start_time = ""
         self.calories= 0
         self.tree = None
         if filename != None:
@@ -99,6 +104,7 @@ class Gpx:
                 trackSegTag = mainNS.substitute(tag="trkseg")
                 elevationTag = mainNS.substitute(tag="ele")
                 nameTag = mainNS.substitute(tag="name")
+                intensityTag = mainNS.substitute(tag="intensity")
 
             logging.debug("getting values...")
             self.Values = self._getValues()
@@ -123,7 +129,8 @@ class Gpx:
             timeResult = trk.find(timeTag)
             if timeResult is not None:
                 time_ = timeResult.text # check timezone
-                mk_time = self.getDateTime(time_)[0]
+                logging.debug("TimeResult: %s" %time_)
+                mk_time = getDateTime(time_)[0]
                 time_ = mk_time.strftime("%Y-%m-%d")
             else:
                 time_ = _("No Data")
@@ -131,9 +138,6 @@ class Gpx:
             tracks.append((name,time_))
         return tracks
 
-    def getDateTime(self, time_):
-        return Date().getDateTime(time_)
-
     def getUnevenness(self):
         return self.upositive,self.unegative
 
@@ -146,6 +150,9 @@ class Gpx:
     def getCalories(self):
         return self.calories
 
+    def getStart_time(self):
+        return self.start_time
+
     def getLaps(self):
         logging.debug(">>")
         lapInfo = []
@@ -160,8 +167,8 @@ class Gpx:
             stLon = self.trkpoints[0]['lon']
             lat = self.trkpoints[-1]['lat']
             lon = self.trkpoints[-1]['lon']
-            #print ((self.total_time, lat, lon, self.calories, self.total_dist, stLat, stLon))
-            lapInfo.append((self.total_time, lat, lon, self.calories, self.total_dist*1000, stLat, stLon))
+            logging.debug("total_time: %s" %self.total_time)
+            lapInfo.append((self.total_time, lat, lon, self.calories, self.total_dist*1000, stLat, stLon, "active", self.hr_average, self.maxhr, self.maxvel, "manual"))
         else:
             for lap in laps:
                 endPoint = lap.find(endPointTag)
@@ -176,12 +183,40 @@ class Gpx:
                 elapsedTime = lap.findtext(elapsedTimeTag)
                 if elapsedTime.count(":") == 2: # got a 0:41:42.14 type elasped time
                     hours, mins, secs = elapsedTime.split(":")
-                    elapsedTime = (int(hours) *3600) + (int(mins) * 60) + float(secs)
-                    #print elapsedTime
+                    elapsedTime = str((int(hours) *3600) + (int(mins) * 60) + float(secs))
                 calories = lap.findtext(calorieTag)
                 distance = lap.findtext(distanceTag)
-                logging.debug("Found time: %s, lat: %s lon: %s cal: %s dist: %s " % (elapsedTime, lat, lon, calories, distance))
-                lapInfo.append((elapsedTime, lat, lon, calories, distance, stLat, stLon))
+                logging.info("Found time: %s, start lat: %s, start lon: %s, end lat: %s end lon: %s cal: %s dist: %s " % (elapsedTime, stLat, stLon, lat, lon, calories, distance))
+                intensity = lap.findtext(intensityTag).lower()
+                trigger_element = lap.find(triggerTag)
+                #logging.debug("Trigger element: %s" % etree.tostring(trigger_element))
+                if "kind" in trigger_element.keys():
+                    trigger = trigger_element.get("kind").lower()
+                else:
+                    trigger = trigger_element.text.lower()
+                # attribute xpath features are only available in 1.3 (python >= 2.7) - 2012/01/28 - dgranda
+                lap_summary = lap.findall(summaryTag)
+                num_elements = len(lap_summary)
+                summary_dict = {"MaximumSpeed": 0,
+                                "AverageHeartRateBpm": None,
+                                "MaximumHeartRateBpm": None}
+                if num_elements > 0:
+                    if num_elements == 1: # old _non compliant_ pytrainer estructure
+                        #logging.debug("lap_summary[0]: %s" % etree.tostring(lap_summary[0]))
+                        for key in summary_dict.keys():
+                            summary_dict[key] = lap_summary[0].findtext(mainNS.substitute(tag=key))
+                            logging.debug("%s: %s" % (key, summary_dict[key]))
+                    else:
+                        for summary_element in lap_summary:
+                            summary_dict[summary_element.get('name')] = summary_element.text
+                            logging.debug("%s: %s" % (summary_element.get('name'), summary_element.text))
+                    max_speed = summary_dict["MaximumSpeed"]
+                    avg_hr = summary_dict["AverageHeartRateBpm"]
+                    max_hr = summary_dict["MaximumHeartRateBpm"]
+                else:
+                    logging.info("No summary found")
+                logging.info("Intensity: %s | Trigger: %s | Max speed: %s | Average hr: %s | Maximum hr: %s" % (intensity, trigger, max_speed, avg_hr, max_hr))
+                lapInfo.append((elapsedTime, lat, lon, calories, distance, stLat, stLon, intensity, avg_hr, max_hr, max_speed, trigger))
         logging.debug("<<")
         return lapInfo
 
@@ -192,13 +227,30 @@ class Gpx:
         logging.debug(">>")
         tree  = self.tree
         # Calories data comes within laps. Maybe more than one, adding them together - dgranda 20100114
+        # Distance data comes within laps where present as well - dgranda 20110204
         laps = tree.findall(lapTag)
         if laps is not None and laps != "":
+            totalDistance = 0
+            totalDuration = 0
             for lap in laps:
-                lapCalories = lap.findtext(calorieTag)
-                logging.debug("Lap calories: "+str(lapCalories))
+                lapCalories = lap.findtext(calorieTag)                
                 self.calories += int(lapCalories)
-            logging.debug("Calories: "+str(self.calories))
+                lapDistance = lap.findtext(distanceTag)
+                totalDistance += float(lapDistance)
+                lapDuration_tmp = lap.findtext(elapsedTimeTag)
+                # When retrieving data from TCX file -> seconds (float)
+                # When retrieving data from GPX+ file -> hh:mm:ss
+                # EAFP -> http://docs.python.org/glossary.html
+                try:
+                    lapDuration = float(lapDuration_tmp)
+                except ValueError:
+                    hour,minu,sec = lapDuration_tmp.split(":")
+                    lapDuration = float(sec) + int(minu)*60 + int(hour)*3600
+                totalDuration += lapDuration 
+                logging.info("Lap distance: %s m | Duration: %s s | Calories: %s kcal" % (lapDistance, lapDuration, lapCalories))
+            self.total_dist = float(totalDistance/1000.0) # Returning km
+            self.total_time = int(totalDuration) # Returning seconds
+            logging.info("Laps - Distance: %.02f km | Duration: %d s | Calories: %s kcal" % (self.total_dist, self.total_time, self.calories))
         else:
             laps = []
 
@@ -219,11 +271,15 @@ class Gpx:
         logging.debug("%d trkpoints in file" % len(trkpoints))
 
         date_ = tree.find(timeTag).text
-        #mk_time = self.getDateTime(date_)[0] #UTC Date
-        mk_time = self.getDateTime(date_)[1] #Local Date
-        self.date = mk_time.strftime("%Y-%m-%d")
+        if date_ is None:
+            logging.info("time tag is blank")
+            self.date = None
+        else:
+            mk_time = getDateTime(date_)[1] #Local Date
+            self.date = mk_time.strftime("%Y-%m-%d")
+            self.start_time = mk_time.strftime("%H:%M:%S")
         waiting_points = []
-
+        logging.debug("date: %s | start_time: %s | mk_time: %s" % (self.date, self.start_time, mk_time))
 
         for i, trkpoint in enumerate(trkpoints):
             #Get data from trkpoint
@@ -252,20 +308,29 @@ class Gpx:
                 cadence = int(cadResult.text)
             else:
                 cadence = None
+
             #get the time
             timeResult = trkpoint.find(timeTag)
             if timeResult is not None:
                 date_ = timeResult.text
-                mk_time = self.getDateTime(date_)[0]
+                mk_time = getDateTime(date_)[0]
                 time_ = time.mktime(mk_time.timetuple()) #Convert date to seconds
                 if i == 0:
                     time_elapsed = 0
                 else:
                     time_elapsed = time_ - self.trkpoints[i-1]['time'] if self.trkpoints[i-1]['time'] is not None else 0
-                    self.total_time += time_elapsed
+                    if time_elapsed > 10:
+                        logging.debug("%d seconds from last trkpt, someone took a break!" % time_elapsed)
+                        # Calculating average lapse between trackpoints to add it
+                        average_lapse = round(self.total_time_trkpts/i)
+                        logging.debug("Adding %d seconds (activity average) as lapse from last point" % average_lapse)
+                        self.total_time_trkpts += average_lapse
+                    else:
+                        self.total_time_trkpts += time_elapsed
             else:
                 time_ = None
                 time_elapsed = None
+
             #get the elevation
             eleResult = trkpoint.find(elevationTag)
             rel_alt = 0
@@ -312,7 +377,7 @@ class Gpx:
             #Accumulate distances
             if dist is not None:
                 dist_elapsed += dist #TODO fix
-                total_dist += dist
+                self.total_dist_trkpts += dist
 
             #Calculate speed...
             vel = self._calculate_speed(dist, time_elapsed, smoothing_factor=3)
@@ -327,14 +392,15 @@ class Gpx:
                         w_time = (w_dist/dist_elapsed) * time_elapsed
                         w_vel = w_dist/((w_time)/3600.0)
                         w_total_time += w_time
+                        logging.info("Time added: %f" % w_time)
                         retorno.append((w_total_dist, w_alt, w_total_time, w_vel, w_lat, w_lon, w_hr, w_cadence, w_corEle))
                     waiting_points = []
                     dist_elapsed = 0
                 else:
-                    retorno.append((total_dist,ele, self.total_time,vel,lat,lon,hr,cadence,corEle))
+                    retorno.append((self.total_dist_trkpts,ele, self.total_time,vel,lat,lon,hr,cadence,corEle))
                     dist_elapsed = 0
             else: # time_ is None
-                waiting_points.append((total_dist, dist_elapsed, ele, self.total_time, lat, lon, hr, cadence, corEle))
+                waiting_points.append((self.total_dist_trkpts, dist_elapsed, ele, self.total_time, lat, lon, hr, cadence, corEle))
 
             #Add to dict of values to trkpoint list
             self.trkpoints.append({ 'id': i,
@@ -344,11 +410,11 @@ class Gpx:
                                     'cadence':cadence,
                                     'time':time_,
                                     'time_since_previous': time_elapsed,
-                                    'time_elapsed': self.total_time,
+                                    'time_elapsed': self.total_time_trkpts,
                                     'ele':ele,
                                     'ele_change': rel_alt,
                                     'distance_from_previous': dist,
-                                    'elapsed_distance': total_dist,
+                                    'elapsed_distance': self.total_dist_trkpts,
                                     'velocity':vel,
                                     'correctedElevation':corEle,
 
@@ -360,7 +426,18 @@ class Gpx:
         self.hr_average = 0
         if len_validhrpoints > 0:
             self.hr_average = total_hr/len_validhrpoints
-        self.total_dist = total_dist
+        # In case there is no other way to calculate distance, we rely on trackpoints (number of trackpoints is configurable!)
+        if self.total_dist is None or self.total_dist == 0:
+            self.total_dist = self.total_dist_trkpts
+        else:
+            dist_diff = 1000*(self.total_dist_trkpts - self.total_dist)
+            logging.debug("Distance difference between laps and trkpts calculation: %f m" % dist_diff)
+        if self.total_time is None or self.total_time == 0:
+            self.total_time = self.total_time_trkpts
+        else:
+            time_diff = self.total_time_trkpts - self.total_time
+            logging.debug("Duration difference between laps and trkpts calculation: %d s" % time_diff)
+        logging.info("Values - Distance: %.02f km | Duration: %d s | Calories: %s kcal" % (self.total_dist, self.total_time, self.calories))
         logging.debug("<<")
         return retorno
 
@@ -435,7 +512,7 @@ class Gpx:
         if date_time is None:
             print "Problems when retrieving start time from "+gpxFile+". Please check data integrity"
             return 0
-        dateTime = self.getDateTime(date_time.text)
+        dateTime = getDateTime(date_time.text)
         zuluDateTime = dateTime[0].strftime("%Y-%m-%dT%H:%M:%SZ")
         localDateTime = dateTime[1]
         logging.debug(gpxFile+" | "+ date_time.text +" | " + zuluDateTime + " | " + str(localDateTime))
