@@ -17,22 +17,15 @@
 #along with this program; if not, write to the Free Software
 #Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-import locale
 import sys
 import os
-import pygtk
-import gobject
-pygtk.require('2.0')
-import gtk
-import gtk.glade
 from optparse import OptionParser
 import logging
 import logging.handlers
 import traceback
 
-from os import path
-
-from pytrainer import platform
+from pytrainer.util.date import DateRange
+from upgrade.data import initialize_data
 from environment import Environment
 from record import Record
 from waypoint import Waypoint
@@ -40,76 +33,87 @@ from extension import Extension
 from importdata import Importdata
 from plugins import Plugins
 from profile import Profile
+from pytrainer.core.sport import SportService
 from athlete import Athlete
+from stats import Stats
 
-from gui.windowimportdata import WindowImportdata
 from gui.windowmain import Main
 from gui.warning import Warning
-from lib.date import Date
-from activitypool import ActivityPool
+from lib.date import Date, getNameMonth
+from pytrainer.core.activity import ActivityService
 from lib.ddbb import DDBB
 from lib.uc import UC
 
 class pyTrainer:
     def __init__(self,filename = None, data_path = None):
-        #Version constants
-        self.version ="1.8.0"
-        self.DB_version = 6
+        # Based on Django's approach -> http://code.djangoproject.com/svn/django/trunk/django/__init__.py
+        self.version = __import__('pytrainer').get_version()
         #Process command line options
         self.startup_options = self.get_options()
-        self.environment = Environment(platform.get_platform(), self.startup_options.conf_dir)
-        self.environment.create_directories()
         #Setup logging
-        self.set_logging(self.startup_options.log_level)
+        self.environment = Environment(self.startup_options.conf_dir, data_path)
+        self.environment.create_directories()
+        self.environment.clear_temp_dir()
+        self.set_logging(self.startup_options.log_level, self.startup_options.log_type)
         logging.debug('>>')
-        logging.debug("PyTrainer version %s, DB version %s" % (self.version, self.DB_version))
+        logging.info("pytrainer version %s" % (self.version))
         self.data_path = data_path
-        self.date = Date()
-        self.ddbb = None
+
         # Checking profile
         logging.debug('Checking configuration and profile...')
-        self.profile = Profile(self.environment, self.data_path,self)
+        self.profile = Profile()
+        # Write the default config to disk
+        self.profile.saveProfile()
         self.uc = UC()
-        self.windowmain = None
-        self.ddbb = DDBB(self.profile, self)
+        self.profilewindow = None
+        self.ddbb = DDBB(self.profile)
         logging.debug('connecting to DDBB')
         self.ddbb.connect()
 
-        #Get user's DB version
-        currentDB_version = self.profile.getValue("pytraining","DB_version")
-        logging.debug("Current DB version: "+str(currentDB_version))
-        # DB check can be triggered either via new version (mandatory) or as runtime parameter (--check)
-        if self.startup_options.check:
-            logging.debug("Checking DB as per user's request")
-            self.sanityCheck()
-        elif currentDB_version is None:
-            logging.debug("No stored DB version. Checking DB")
-            self.sanityCheck()
-        elif self.DB_version > int(currentDB_version):
-            logging.debug("DB version newer than user's version")
-            self.sanityCheck()
-        else:
-            logging.info('No sanity check requested')
-        self.record = Record(data_path,self)
-        self.athlete = Athlete(data_path,self)
+        logging.info('Checking if some upgrade action is needed...')
+        initialize_data(self.ddbb, self.environment.conf_dir)
+
+        # Loading shared services
+        logging.debug('Loading sport service...')
+        self._sport_service = SportService(self.ddbb)
+        logging.debug('Loading record service...')
+        self.record = Record(self._sport_service, data_path, self)
+        logging.debug('Loading athlete service...')
+        self.athlete = Athlete(data_path, self)
+        logging.debug('Loading stats service...')
+        self.stats = Stats(self._sport_service, self)
+        logging.debug('Initializing activity pool...')
         pool_size = self.profile.getIntValue("pytraining","activitypool_size", default=1)
-        self.activitypool = ActivityPool(self, size=pool_size)
-        #preparamos la ventana principal
-        self.windowmain = Main(data_path,self,self.version, gpxDir=self.profile.gpxdir)
-        self.date = Date(self.windowmain.calendar)
+        self.activitypool = ActivityService(self, size=pool_size)
+
+        #Loading main window
+        self.windowmain = None
+        logging.debug('Loading main window...')
+        self.windowmain = Main(self._sport_service, data_path,self,self.version, gpxDir=self.profile.gpxdir)
+
+        # Select initial date depending on user's preference
+        self.selectInitialDate()
+        
+        logging.debug('Loading waypoint service...')
         self.waypoint = Waypoint(data_path,self)
+        logging.debug('Loading extension service...')
         self.extension = Extension(data_path, self)
+        logging.debug('Loading plugins service...')
         self.plugins = Plugins(data_path, self)
-        self.importdata = Importdata(data_path, self, self.profile)
+        self.importdata = Importdata(self._sport_service, data_path, self, self.profile)
+        logging.debug('Loading plugins...')
         self.loadPlugins()
+        logging.debug('Loading extensions...')
         self.loadExtensions()
+        logging.debug('Setting values for graphs, maps and waypoint editor...')
         self.windowmain.setup()
         self.windowmain.on_calendar_selected(None)
+        logging.debug('Refreshing sport list... is this needed?')
         self.refreshMainSportList()
+        logging.debug('Launching main window...')
         self.windowmain.run()
         logging.debug('<<')
 
-
     def get_options(self):
         '''
         Define usage and accepted options for command line startup
@@ -121,24 +125,29 @@ class pyTrainer:
         For more help on valid options try:
            %prog -h '''
         parser = OptionParser(usage=usage)
-        parser.set_defaults(log_level=logging.ERROR, validate=False, equip=False, newgraph=True, conf_dir=None)
+        parser.set_defaults(log_level=logging.WARNING, validate=False, equip=False, newgraph=True, conf_dir=None, log_type="file")
         parser.add_option("-d", "--debug", action="store_const", const=logging.DEBUG, dest="log_level", help="enable logging at debug level")
         parser.add_option("-i", "--info", action="store_const", const=logging.INFO, dest="log_level", help="enable logging at info level")
         parser.add_option("-w", "--warn", action="store_const", const=logging.WARNING, dest="log_level", help="enable logging at warning level")
+        parser.add_option("--error", action="store_const", const=logging.ERROR,
+                          dest="log_level", help="enable logging at error level")
         parser.add_option("--valid", action="store_true", dest="validate", help="enable validation of files imported by plugins (details at info or debug logging level) - note plugin must support validation")
-        parser.add_option("--check", action="store_true", dest="check", help="triggers database (only sqlite based) and configuration file sanity checks, adding fields if necessary. Backup of database is done before any change. Details at info or debug logging level")
         parser.add_option("--oldgraph", action="store_false", dest="newgraph", help="Turn off new graphing approach")
         parser.add_option("--newgraph", action="store_true", dest="newgraph", help="Deprecated Option: Turn on new graphing approach")
         parser.add_option("--confdir", dest="conf_dir", help="Specify the directory where application configuration will be stored.")
+        parser.add_option("--logtype", dest="log_type", metavar="TYPE",  type="choice" , choices=["file", "console"], help="Specify where logging should be output to. TYPE is one of 'file' (default), or 'console'.")
         (options, args) = parser.parse_args()
         return options
 
-    def set_logging(self,level):
+    def set_logging(self, level, log_type):
         '''Setup rotating log file with customized format'''
-        rotHandler = logging.handlers.RotatingFileHandler(self.environment.log_file, maxBytes=100000, backupCount=5)
+        if("console" == log_type):
+            handler = logging.StreamHandler(sys.stdout)
+        else:
+            handler = logging.handlers.RotatingFileHandler(self.environment.log_file, maxBytes=100000, backupCount=5)
         formatter = logging.Formatter('%(asctime)s|%(levelname)s|%(module)s|%(funcName)s|%(message)s')
-        rotHandler.setFormatter(formatter)
-        logging.getLogger('').addHandler(rotHandler)
+        handler.setFormatter(formatter)
+        logging.getLogger('').addHandler(handler)
         self.set_logging_level(self.startup_options.log_level)
 
     def set_logging_level(self, level):
@@ -152,7 +161,25 @@ class pyTrainer:
         #self.webservice.stop()
         self.windowmain.gtk_main_quit()
         logging.shutdown()
-        sys.exit() # Any nonzero value is considered “abnormal termination” by shells and the like
+        sys.exit() # Any nonzero value is considered "abnormal termination" by shells and the like
+
+    def selectInitialDate(self):
+        logging.debug('>>')
+        # self.windowmain.calendar comes from SimpleGladeApp initialisation, not really sure how... :?
+        self.date = Date(self.windowmain.calendar)
+        if self.profile.getValue("pytraining","prf_startscreen") == "last_entry":
+            logging.info("User selection is to display last entry in start screen")
+            last_entry_date = self.record.getLastRecordDateString()
+            try:
+                logging.info("Last activity found on %s" %last_entry_date)
+                self.date.setDate(last_entry_date)
+            except:
+                logging.error("No data available regarding last activity date. Default date will be today")
+                logging.debug("Traceback: %s" % traceback.format_exc())
+        else:
+            logging.info("User selection is to display current day in start screen")
+        logging.debug('Setting date to %s' % self.date.getDate().strftime("%Y-%m-%d"))
+        logging.debug('<<')
 
     def loadPlugins(self):
         logging.debug('>>')
@@ -161,8 +188,14 @@ class pyTrainer:
              logging.info("No active plugins")
         else:
              for plugin in activeplugins:
-                txtbutton = self.plugins.loadPlugin(plugin)
-                self.windowmain.addImportPlugin(txtbutton)
+                # From version 1.10 on all file imports are managed via File -> import wizard
+                # Only showing garmintools_full and garmin-hr in 'File' dropdown
+                plugin_name = os.path.basename(plugin)
+                if (plugin_name == "garmintools_full" or plugin_name == "garmin-hr"):
+                    txtbutton = self.plugins.loadPlugin(plugin)
+                    self.windowmain.addImportPlugin(txtbutton)
+                else:
+                    logging.debug('From version 1.10 on, file import plugins are managed via File -> Import. Not displaying plugin ' + plugin_name)
         logging.debug('<<')
 
     def loadExtensions(self):
@@ -211,8 +244,8 @@ class pyTrainer:
 
     def refreshMainSportList(self):
         logging.debug('>>')
-        listSport = self.profile.getSportList()
-        self.windowmain.updateSportList(listSport)
+        sports = self._sport_service.get_all_sports()
+        self.windowmain.updateSportList(sports)
         logging.debug('<<')
 
     def refreshGraphView(self, view, sport=None):
@@ -232,6 +265,8 @@ class pyTrainer:
                 self.refreshRecordGraphView("map")
              elif self.windowmain.recordview.get_current_page()==3:
                 self.refreshRecordGraphView("heartrate")
+             elif self.windowmain.recordview.get_current_page()==4:
+                self.refreshRecordGraphView("analytics")
         elif view=="day":
              logging.debug('day view')
              sport = self.windowmain.activeSport
@@ -241,28 +276,27 @@ class pyTrainer:
              #selected,iter = self.windowmain.recordTreeView.get_selection().get_selected()
         elif view=="week":
              logging.debug('week view')
-             date_ini, date_end = self.date.getWeekInterval(date_selected, self.profile.prf_us_system)
+             date_range = DateRange.for_week_containing(date_selected)
              sport = self.windowmain.activeSport
              sport_id = self.record.getSportId(sport)
-             record_list = self.record.getrecordPeriod(date_ini, date_end, sport_id)
-             self.windowmain.actualize_weekview(record_list, date_ini, date_end)
+             record_list = self.record.getrecordPeriod(date_range, sport_id)
+             self.windowmain.actualize_weekview(record_list, date_range)
         elif view=="month":
              logging.debug('month view')
-             date_ini, date_end = self.date.getMonthInterval(date_selected)
+             date_range = DateRange.for_month_containing(date_selected)
              sport = self.windowmain.activeSport
              sport_id = self.record.getSportId(sport)
-             record_list = self.record.getrecordPeriodSport(date_ini, date_end,sport_id)
-             nameMonth, daysInMonth = self.date.getNameMonth(date_selected)
+             record_list = self.record.getrecordPeriod(date_range, sport_id)
+             nameMonth, daysInMonth = getNameMonth(date_selected)
              self.windowmain.actualize_monthview(record_list, nameMonth)
              self.windowmain.actualize_monthgraph(record_list, daysInMonth)
         elif view=="year":
              logging.debug('year view')
-             date_ini, date_end = self.date.getYearInterval(date_selected)
+             date_range = DateRange.for_year_containing(date_selected)
              sport = self.windowmain.activeSport
              sport_id = self.record.getSportId(sport)
-             year = self.date.getYear(date_selected)
-             record_list = self.record.getrecordPeriodSport(date_ini, date_end,sport_id)
-             self.windowmain.actualize_yearview(record_list, year)
+             record_list = self.record.getrecordPeriod(date_range, sport_id)
+             self.windowmain.actualize_yearview(record_list, date_selected.year)
              self.windowmain.actualize_yeargraph(record_list)
         elif view=="listview":
             logging.debug('list view')
@@ -270,10 +304,13 @@ class pyTrainer:
         elif view=="athlete":
             logging.debug('athlete view')
             self.windowmain.on_athleteview_activate()
+        elif view=="stats":
+            logging.debug('stats view')
+            self.windowmain.on_statsview_activate()
         else:
             print "Unknown view %s" % view
         logging.debug('<<')
-
+        
     def refreshRecordGraphView(self, view, id_record=None):
         logging.debug('>>')
         logging.info('Working on '+view+' graph')
@@ -302,6 +339,8 @@ class pyTrainer:
         if view=="heartrate":
              self.windowmain.actualize_heartrategraph(activity)
              self.windowmain.actualize_hrview(activity)
+        if view=="analytics":
+             self.windowmain.actualize_analytics(activity)
         logging.debug('<<')
 
     def refreshMapView(self, full_screen=False):
@@ -319,9 +358,6 @@ class pyTrainer:
 
     def refreshListRecords(self):
         logging.debug('>>')
-        #Refresh list view
-        #self.refreshListView() # old variant
-        self.refreshListView(self.windowmain.listsearch.condition)
         #Refresh list records
         date = self.date.getDate()
         sport = self.windowmain.activeSport
@@ -339,6 +375,12 @@ class pyTrainer:
         self.windowmain.actualize_athleteview(self.athlete)
         logging.debug('<<')
 
+    def refreshStatsView(self):
+        logging.debug('>>')
+        self.stats.refresh()
+        self.windowmain.actualize_statsview(self.stats, self.record.getAllRecordList())
+        logging.debug('<<')
+
     def refreshListView(self,condition=None):
         logging.debug('>>')
         record_list = self.record.getRecordListByCondition(condition)
@@ -391,8 +433,12 @@ class pyTrainer:
         for plugin in activeplugins_after:
             if plugin not in activeplugins_before:
                 #new active plugin -> need to load plugin
-                txtbutton = self.plugins.loadPlugin(plugin)
-                self.windowmain.addImportPlugin(txtbutton)
+                plugin_name = os.path.basename(plugin)
+                if (plugin_name == "garmintools_full" or plugin_name == "garmin-hr"):
+                    txtbutton = self.plugins.loadPlugin(plugin)
+                    self.windowmain.addImportPlugin(txtbutton)
+                else:
+                    logging.debug('From version 1.10 on file import plugins are managed via File -> Import. Not displaying plugin ' + plugin_name)
         logging.debug('<<')
 
     def setExtensions(self, before, after):
@@ -413,22 +459,18 @@ class pyTrainer:
                 self.windowmain.addExtension(txtbutton)
         logging.debug('<<')
 
-    def newRecord(self,title=None,distance=None,time=None,upositive=None,unegative=None,bpm=None,calories=None,date=None,comment=None,view=None):
+    def newRecord(self,title=None,distance=None,time=None,upositive=None,unegative=None,bpm=None,calories=None,comment=None,view=None):
         logging.debug('>>')
-        list_sport = self.profile.getSportList()
-        if date == None:
-             date = self.date.getDate()
-        self.record.newRecord(list_sport, date, title, distance, time, upositive, unegative, bpm, calories, comment)
+        date = self.date.getDate()
+        self.record.newRecord(date, title, distance, time, upositive, unegative, bpm, calories, comment)
         self.refreshListRecords()
         if view is not None:
             self.refreshGraphView(view)
         logging.debug('<<')
 
     def editRecord(self, id_record, view=None):
-        logging.debug('>>')
-        list_sport = self.profile.getSportList()
-        logging.debug('id_record: '+str(id_record)+' | list_sport: '+str(list_sport))
-        self.record.editRecord(id_record,list_sport)
+        logging.debug("Editing record with id: '%s'", id_record)
+        self.record.editRecord(id_record)
         self.refreshListRecords()
         if view is not None:
             self.refreshGraphView(view)
@@ -477,19 +519,19 @@ class pyTrainer:
 
     def editProfile(self):
         logging.debug('>>')
-        self.profile.editProfile()
+        from gui.windowprofile import WindowProfile
+        self.profile.refreshConfiguration()
+        if self.profilewindow is None:
+            self.profilewindow = WindowProfile(self._sport_service, self.data_path, self.profile, pytrainer_main=self)
+            logging.debug("setting data values")
+            self.profilewindow.setValues(self.profile.configuration)
+            self.profilewindow.run()
+            self.profilewindow = None
+        else:
+            self.profilewindow.setValues(self.profile.configuration)
+            self.profilewindow.present()
+        self.profile.refreshConfiguration()
+
         self.activitypool.clear_pool()
         self.windowmain.setup()
         logging.debug('<<')
-
-    def sanityCheck(self):
-        """23.11.2009 - dgranda
-        Checks database and configuration file
-        args: none
-        returns: none"""
-        logging.debug('>>')
-        logging.info('Checking database integrity')
-        self.ddbb.checkDBIntegrity()
-        logging.info('Setting DB version to: ' + str(self.DB_version))
-        self.profile.setValue("pytraining","DB_version", str(self.DB_version))
-        logging.debug('<<')
