@@ -22,26 +22,17 @@ import logging
 from StringIO import StringIO
 
 from lxml import etree
-from lib.ddbb import DDBB
+from environment import Environment
+from pytrainer.lib.singleton import Singleton
 from lib.uc import UC
 
-class Profile:
-    def __init__(self, environment, data_path = None, parent = None):
+class Profile(Singleton):
+    def __init__(self):
         logging.debug(">>")
-        self.environment = environment
-        self.pytrainer_main = parent
-        self.data_path = data_path
-        self.xml_tree = None
-        self.tmpdir = environment.temp_dir
-        self.confdir = environment.conf_dir
-        self.conffile = environment.conf_file
-        self.gpxdir = environment.gpx_dir
-        self.extensiondir = environment.extension_dir
-        self.plugindir = environment.plugin_dir
+        # The first two variables are singletons themselves, profile_options
+        # doesn't change, only xml_tree needs to be protected by has_attr.
+        self.environment = Environment()
         self.uc = UC()
-        
-        environment.clear_temp_dir()
-
         #Profile Options and Defaults
         self.profile_options = {
             "prf_name":"default",
@@ -55,7 +46,6 @@ class Profile:
             "prf_ddbbuser":"",
             "prf_ddbbpass":"",
             "version":"0.0",
-            "DB_version":"0",
             "prf_us_system":"False",
             "prf_hrzones_karvonen":"False",
             "prf_maxhr":"",
@@ -65,13 +55,48 @@ class Profile:
             "default_viewer":"0",
             "window_size":"800, 640",
             "activitypool_size": "10",
+            "prf_startscreen":"current_day",
             }
 
-        #Parse pytrainer configuration file
-        self.config_file = environment.conf_file
+        if not hasattr(self, 'xml_tree'):
+            self.xml_tree = None
+            #Parse pytrainer configuration file
+            self.refreshConfiguration()
+
+        logging.debug("<<")
+
+    @property
+    def data_path(self):
+        return self.environment.data_path
+
+    @property
+    def tmpdir(self):
+        return self.environment.temp_dir
+
+    @property
+    def confdir(self):
+        return self.environment.conf_dir
+
+    @property
+    def config_file(self):
+        return self.environment.conf_file
+
+    @property
+    def gpxdir(self):
+        return self.environment.gpx_dir
+
+    @property
+    def extensiondir(self):
+        return self.environment.extension_dir
+
+    @property
+    def plugindir(self):
+        return self.environment.plugin_dir
+
+    def refreshConfiguration(self):
+        logging.debug(">>")
         self.configuration = self._parse_config_file(self.config_file)
-        logging.debug("Configuration retrieved: "+str(self.configuration))
-        #self.pytrainer_main.ddbb = DDBB(self, pytrainer_main=self.pytrainer_main)
+        logging.debug("Configuration retrieved: %s", str(self.configuration))
         self.uc.set_us(self.prf_us_system)
         self._setZones()
         logging.debug("<<")
@@ -81,10 +106,15 @@ class Profile:
         #resthr = self.getValue("pytraining","prf_minhr")
         try:
             maxhr = int(self.getValue("pytraining","prf_maxhr"))
+        except Exception as e:
+            logging.error("Failed when retrieving Max Heartrate value: %s" %e)
+            logging.info("Setting Max Heartrate to default value: 190")
+            maxhr = 190
+        try:
             resthr = int(self.getValue("pytraining","prf_minhr"))
         except Exception as e:
-            logging.debug(str(e))
-            maxhr = 220
+            logging.error("Failed when retrieving Min Heartrate value: %s" %e)
+            logging.info("Setting Min Heartrate to default value: 65")
             resthr = 65
         self.maxhr = maxhr
         self.rethr = resthr
@@ -121,12 +151,6 @@ class Profile:
     def getZones(self):
         return self.zone5,self.zone4,self.zone3,self.zone2,self.zone1
 
-    def getConfFile(self):
-        if not os.path.isfile(self.conffile):
-            return False
-        else:
-            return self.conffile
-
     def _parse_config_file(self, config_file):
         '''
         Parse the xml configuration file and convert to a dict
@@ -137,40 +161,39 @@ class Profile:
             logging.error("Configuration file value not set")
             logging.error("Fatal error, exiting")
             exit(-3)
-        if not os.path.isfile(config_file): #File not found
+        elif not os.path.isfile(config_file): #File not found
             logging.error("Configuration '%s' file does not exist" % config_file)
             logging.info("No profile found. Creating default one")
             self.setProfile(self.profile_options)
-        if os.stat(config_file)[stat.ST_SIZE] == 0: #File is empty
+        elif os.stat(config_file)[stat.ST_SIZE] == 0: #File is empty
             logging.error("Configuration '%s' file is empty" % config_file)
             logging.info("Creating default profile")
             self.setProfile(self.profile_options)
-        logging.debug("Attempting to parse content from "+ config_file)
-        try:
+        else:
+            logging.debug("Attempting to parse content from %s", config_file)
             parser = etree.XMLParser(encoding='UTF8', recover=True)
-            self.xml_tree = etree.parse(config_file, parser=parser)
-            #Have a populated xml tree, get pytraining node (root) and convert it to a dict
-            pytraining_tag = self.xml_tree.getroot()
-            config = {}
-            config_needs_update = False
-            for key, default in self.profile_options.items():
-                value = pytraining_tag.get(key)
-                #If property is not found, set it to the default
-                if value is None:
-                    config_needs_update = True
-                    value = default
-                config[key] = value
-            #Added a property, so update config
-            if config_needs_update:
-                self.setProfile(config)
-            #Set shorthand var for units of measurement
-            self.prf_us_system = True if config["prf_us_system"] == "True" else False
-            return config
-        except Exception as e:
-            logging.error("Error parsing file: %s. Exiting" % config_file)
-            logging.error(str(e))
-            logging.error("Fatal error, exiting")
-            exit(-3)
+            try:
+                self.xml_tree = etree.parse(config_file, parser=parser)
+            except Exception as e:
+                logging.error("Error %s while parsing file %s. Exiting", e, config_file)
+                exit(-3)
+        #Have a populated xml tree, get pytraining node (root) and convert it to a dict
+        pytraining_tag = self.xml_tree.getroot()
+        config = {}
+        config_needs_update = False
+        for key, default in self.profile_options.items():
+            value = pytraining_tag.get(key)
+            #If property is not found, set it to the default
+            if value is None:
+                config_needs_update = True
+                value = default
+            config[key] = value
+        #Added a property, so update config
+        if config_needs_update:
+            self.setProfile(config)
+        #Set shorthand var for units of measurement
+        self.prf_us_system = True if config["prf_us_system"] == "True" else False
+        return config
 
     def getIntValue(self, tag, variable, default=0):
         ''' Function to return conf value as int
@@ -199,15 +222,14 @@ class Profile:
     def setValue(self, tag, variable, value, delay_write=False):
         logging.debug(">>")
         if tag != "pytraining":
-            print "ERROR - pytraining is the only profile tag supported"
+            logging.error("ERROR: pytraining is the only profile tag supported")
         logging.debug("Setting %s to %s" % (variable, value))
         if self.xml_tree is None:
             #new config file....
             self.xml_tree = etree.parse(StringIO('''<?xml version='1.0' encoding='UTF-8'?><pytraining />'''))
         self.xml_tree.getroot().set(variable, value.decode('utf-8'))
         if not delay_write:
-            logging.debug("Writting...")
-            self.xml_tree.write(self.config_file, xml_declaration=True, encoding='UTF-8')
+            self.saveProfile()
         logging.debug("<<")
 
     def setProfile(self,list_options):
@@ -215,71 +237,10 @@ class Profile:
         for option, value in list_options.items():
             logging.debug("Adding "+option+"|"+value)
             self.setValue("pytraining",option,value,delay_write=True)
-        self.xml_tree.write(self.config_file, xml_declaration=True, encoding='UTF-8')
         self.uc.set_us(list_options['prf_us_system'])
         logging.debug("<<")
 
-    def getSportList(self):
-        logging.debug("--")
-        #connection = self.pytrainer_main.ddbb.connect()
-        #if (connection == 1):
-        logging.debug("retrieving sports info")
-        return self.pytrainer_main.ddbb.select("sports","name,met,weight,id_sports,max_pace",None)
-        #else:
-        #   return connection
-
-    def addNewSport(self,sport,met,weight,maxpace):
-        """31.08.2008 - dgranda
-        It adds a new sport.
-        arguments:
-            sport: sport's name
-            met:
-            weight:
-        returns: id_sports from new sport"""
-        logging.debug(">>")
-        logging.debug("Adding new sport: "+sport+"|"+weight+"|"+met+"|"+maxpace)
-        sport = [sport,met,weight,maxpace]
-        self.pytrainer_main.ddbb.insert("sports","name,met,weight,max_pace",sport)
-        sport_id = self.pytrainer_main.ddbb.select("sports","id_sports","name=\"%s\"" %(sport))
-        logging.debug("<<")
-        return sport_id
-
-    def delSport(self,sport):
-        logging.debug(">>")
-        condition = "name=\"%s\"" %sport
-        id_sport = self.pytrainer_main.ddbb.select("sports","id_sports",condition)[0][0]
-        logging.debug("removing records from sport "+ sport + " (id_sport: "+str(id_sport)+")")
-        self.pytrainer_main.ddbb.delete("records","sport=\"%d\""%id_sport)
-        self.pytrainer_main.ddbb.delete("sports","id_sports=\"%d\""%id_sport)
-        logging.debug("<<")
-
-    def updateSport(self,oldnamesport,newnamesport,newmetsport,newweightsport,newmaxpace=None):
-        logging.debug("--")
-        self.pytrainer_main.ddbb.update("sports","name,met,weight,max_pace",[newnamesport,newmetsport,newweightsport, newmaxpace],"name=\"%s\""%oldnamesport)
-
-    def getSportInfo(self,namesport):
-        logging.debug("--")
-        return self.pytrainer_main.ddbb.select("sports","name,met,weight,max_pace","name=\"%s\""%namesport)[0]
-
-    def build_ddbb(self):
-        logging.debug("--")
-        self.pytrainer_main.ddbb.build_ddbb()
-
-    def editProfile(self):
-        logging.debug(">>")
-        from gui.windowprofile import WindowProfile
-        logging.debug("retrieving configuration data")
-        #Refresh configuration
-        self.configuration = self._parse_config_file(self.config_file)
-        profilewindow = WindowProfile(self.data_path, self, pytrainer_main=self.pytrainer_main)
-        logging.debug("setting data values")
-        profilewindow.setValues(self.configuration)
-        profilewindow.run()
-        self.configuration = self._parse_config_file(self.config_file)
-        logging.debug("<<")
-
-    def actualize_mainsportlist(self):
-        logging.debug("--")
-        self.pytrainer_main.refreshMainSportList()
-
-
+    def saveProfile(self):
+        logging.debug("Writting configuration...")
+        self.xml_tree.write(self.config_file, xml_declaration=True,
+                            encoding='UTF-8')
