@@ -24,15 +24,16 @@ import traceback
 
 from gui.windowrecord import WindowRecord
 from gui.dialogselecttrack import DialogSelectTrack
-from lib.ddbb import DDBB
 from lib.xmlUtils import XMLParser
-from lib.date import Date
+from lib.date import Date, time2second
 from lib.gpx import Gpx
-from pytrainer.equipment import EquipmentService
+from pytrainer.core.equipment import EquipmentService
+from pytrainer.core.sport import Sport
 
 class Record:
-	def __init__(self, data_path = None, parent = None):
+	def __init__(self, sport_service, data_path = None, parent = None):
 		logging.debug('>>')
+		self._sport_service = sport_service
 		self.parent = parent
 		self.pytrainer_main = parent
 		self._equipment_service = EquipmentService(self.pytrainer_main.ddbb)
@@ -41,42 +42,32 @@ class Record:
 		self.date = Date()
 		logging.debug('<<')
 
-	def newRecord(self, list_sport, date, title=None, distance=None, time=None, upositive=None, unegative=None, bpm=None, calories=None, comment=None):
+	def newRecord(self, date, title=None, distance=None, time=None, upositive=None, unegative=None, bpm=None, calories=None, comment=None):
 		logging.debug('>>')
-		self.recordwindow = WindowRecord(self._equipment_service, self.data_path, list_sport,self, date, title, distance, time, upositive, unegative, bpm, calories, comment)
+		sports = self._sport_service.get_all_sports()
+		self.recordwindow = WindowRecord(self._equipment_service, self.data_path, sports, self, self.format_date(date), title, distance, time, upositive, unegative, bpm, calories, comment)
 		self.recordwindow.run()
 		logging.debug('<<')
 
-	def newMultiRecord(self, activities, list_sport):
+	def newMultiRecord(self, activities):
 		logging.debug('>>')
-		#activities (activity_id, start_time, distance, duration, sport, gpx_file)
-		self.recordwindow = WindowRecord(self._equipment_service, self._equipment_service, self.data_path, list_sport, parent=self, windowTitle="Modify details before importing")
+		sports = self._sport_service.get_all_sports()
+		self.recordwindow = WindowRecord(self._equipment_service, self.data_path, sports, parent=self, windowTitle=_("Modify details before importing"))
 		self.recordwindow.populateMultiWindow(activities)
 		self.recordwindow.run()
 		return self.recordwindow.getActivityData()
 		logging.debug('<<')
 
-	def editRecord(self,id_record,list_sport):
+	def editRecord(self,id_record):
 		logging.debug('>>')
 		activity = self.pytrainer_main.activitypool.get_activity(id_record)
 		record_equipment = self.get_record_equipment(id_record)
-		self.recordwindow = WindowRecord(self._equipment_service, self.data_path, list_sport, self, None, windowTitle=_("Edit Entry"), equipment=record_equipment)
+		sports = self._sport_service.get_all_sports()
+		self.recordwindow = WindowRecord(self._equipment_service, self.data_path, sports, self, None, windowTitle=_("Edit Entry"), equipment=record_equipment)
 		self.recordwindow.setValuesFromActivity(activity)
-		'''
-		record = self.pytrainer_main.ddbb.select("records", "id_record, date, sport, distance, time, beats, average, calories, comments, gpslog, title, upositive, unegative, maxspeed, maxpace, pace, maxbeats, date_time_utc, date_time_local", "id_record=\"%s\"" %id_record)
-		logging.debug('retrieving data from DB: '+str(record))
-		gpxfile = self.pytrainer_main.profile.gpxdir+"/%d.gpx"%int(id_record)
-		logging.debug('gpx file associated: '+gpxfile)
-		self.recordwindow = WindowRecord(self.data_path, list_sport,self, None, windowTitle=_("Edit Entry"))
-		if os.path.isfile(gpxfile):
-			self.recordwindow.rcd_gpxfile.set_text(gpxfile)
-			self.recordwindow.frameGeneral.set_sensitive(0)		#Currently record values not changed if a GPX file is present
-			self.recordwindow.frameVelocity.set_sensitive(0)	#Greying out options to indicate this to user
-		logging.debug('sending record info to window')
-		self.recordwindow.setValues(record[0])
-		'''
 		logging.debug('launching window')
 		self.recordwindow.run()
+                self.pytrainer_main.refreshMainSportList()
 		logging.debug('<<')
 
 	def removeRecord(self,id_record):
@@ -90,54 +81,34 @@ class Record:
 			logging.debug('removed gpxfile '+gpxfile)
 		logging.debug('<<')
 
-	def _formatRecord (self, list_options):
-		logging.debug('>>')
-		time = self.date.time2second(list_options["rcd_time"])
-		average = self.parseFloatRecord(list_options["rcd_average"])
-		cells= "date,sport,distance,time,beats,comments,average,calories,title,upositive,unegative,maxspeed,maxpace,pace,maxbeats"
-		if (list_options["rcd_beats"] == ""):
-			list_options["rcd_beats"] = 0
-
-		#calculate the sport id
-		sport_id = self.pytrainer_main.ddbb.select("sports","id_sports","name=\"%s\"" %list_options["rcd_sport"])[0][0]
-
-		values= (
-			list_options["rcd_date"],
-			sport_id,
-			self.parseFloatRecord(list_options["rcd_distance"]),
-			time,
-			self.parseFloatRecord(list_options["rcd_beats"]),
-			list_options["rcd_comments"],
-			average,
-			self.parseFloatRecord(list_options["rcd_calories"]),
-			list_options["rcd_title"],
-			self.parseFloatRecord(list_options["rcd_upositive"]),
-			self.parseFloatRecord(list_options["rcd_unegative"]),
-			self.parseFloatRecord(list_options["rcd_maxvel"]),
-			self.pace_to_float(list_options["rcd_maxpace"]),
-			self.pace_to_float(list_options["rcd_pace"]),
-			self.parseFloatRecord(list_options["rcd_maxbeats"])
-			)
-		logging.debug('<<')
-		return cells,values
-
 	def pace_to_float(self, value):
 		'''Take a mm:ss or mm.ss and return float'''
-		value = value.replace(':', '.')
 		try:
 			value = float(value)
-		except ValueError:
-			value = None
+		except:
+			if ":" in value: # 'mm:ss' found
+				mins, sec = value.split(":")
+				value = float(mins + "." + "%02d" %round(int(sec)*5/3))
+			elif "," in value:
+				value = float(value.replace(',','.'))
+			else:
+				logging.error("Wrong value provided: %s" %value)
+				value = None
 		return value
 
-	def pace_from_float(self, value):
+	def pace_from_float(self, value, fromDB=False):
 		'''Helper to generate mm:ss from float representation mm.ss (or mm,ss?)'''
 		#Check that value supplied is a float
 		try:
 			_value = "%0.2f" % float(value)
 		except ValueError:
 			_value = str(value)
-		return _value.replace('.',':')
+		if fromDB:  # paces in DB are stored in mixed format -> 4:30 as 4.3 (NOT as 4.5 aka 'decimal')
+			pace = _value
+		else:
+			mins, sec_dec = _value.split(".")
+			pace = mins + ":" + "%02d" %round(int(sec_dec)*3/5)
+		return pace
 
 	def _formatRecordNew (self, list_options):
 		"""20.07.2008 - dgranda
@@ -145,7 +116,7 @@ class Record:
 		args: list with keys and values without valid format
 		returns: keys and values matching DB schema"""
 		logging.debug('>>')
-		time = self.date.time2second(list_options["rcd_time"])
+		time = time2second(list_options["rcd_time"])
 		average = self.parseFloatRecord(list_options["rcd_average"])
 		keys= "date,sport,distance,time,beats,comments,average,calories,title,upositive,unegative,maxspeed,maxpace,pace,maxbeats,date_time_utc,date_time_local, duration"
 		if (list_options["rcd_beats"] == ""):
@@ -237,10 +208,36 @@ class Record:
 			tmp_lap['end_lat'] = lap[1]
 			tmp_lap['end_lon'] = lap[2]
 			tmp_lap['calories'] = lap[3]
+			tmp_lap['intensity'] = lap[7]
+			tmp_lap['avg_hr'] = lap[8]
+			tmp_lap['max_hr'] = lap[9]
+			tmp_lap['max_speed'] = lap[10]
+			tmp_lap['laptrigger'] = lap[11]
+			tmp_lap['comments'] = ""
 			laps.append(tmp_lap)
 		logging.debug('<<')
 		return laps
 
+	def hrFromLaps(self, laps):
+		logging.debug('>>')
+		lap_avg_hr = 0 
+		lap_max_hr = 0
+		total_duration = 0
+		ponderate_hr = 0;
+		for lap in laps:
+			if (lap['max_hr'] is None):
+				lap['max_hr'] = 0
+			if (lap['avg_hr'] is None):
+				lap['avg_hr'] = 0
+			if int(lap['max_hr']) > lap_max_hr:
+				lap_max_hr = int(lap['max_hr'])
+			total_duration = total_duration + float(lap['elapsed_time'])
+			ponderate_hr = ponderate_hr + float(lap['elapsed_time'])*int(lap['avg_hr'])
+			logging.debug("Lap number: %s | Duration: %s | Average hr: %s | Maximum hr: %s" % (lap['lap_number'], lap['elapsed_time'], lap['avg_hr'], lap['max_hr']))
+		lap_avg_hr = int(round(ponderate_hr/total_duration)) # ceil?, floor?, round?
+		logging.debug('<<')
+		return lap_avg_hr, lap_max_hr
+
 	def summaryFromGPX(self, gpxOrig, entry):
 		"""29.03.2008 - dgranda
 		Retrieves info which will be stored in DB from GPX file
@@ -291,10 +288,23 @@ class Record:
 			print "#TODO fix record summaryRecord local and utc time..."											#
 		logging.debug('summary: '+str(summaryRecord))
 		laps = self.lapsFromGPX(gpx)
+		# Heartrate data can't be retrieved if no trackpoints present, calculating from lap info
+		lap_avg_hr, lap_max_hr = self.hrFromLaps(laps)
+		logging.debug("HR data from laps. Average: %s | Maximum hr: %s" % (lap_avg_hr, lap_max_hr))
+		if int(summaryRecord['rcd_beats']) > 0:
+			logging.debug("Average heartbeat - Summary: %s | Laps: %s" % (summaryRecord['rcd_beats'], lap_avg_hr))
+		else:
+			logging.debug("No average heartbeat found, setting value (%s) from laps", lap_avg_hr)
+			summaryRecord['rcd_beats'] = lap_avg_hr
+		if int(summaryRecord['rcd_maxbeats']) > 0:
+			logging.debug("Max heartbeat - Summary: %s | Laps: %s" % (summaryRecord['rcd_maxbeats'], lap_max_hr))
+		else:
+			logging.debug("No max heartbeat found, setting value (%s) from laps", lap_max_hr)
+			summaryRecord['rcd_maxbeats'] = lap_max_hr
 		logging.debug('<<')
 		return summaryRecord, laps
 
-	def updateRecord(self, list_options, id_record, equipment=None):
+	def updateRecord(self, list_options, id_record, equipment=None): # ToDo: update only fields that can change if GPX file is present
 		logging.debug('>>')
 		#Remove activity from pool so data is updated
 		self.pytrainer_main.activitypool.remove_activity(id_record)
@@ -305,9 +315,9 @@ class Record:
 				shutil.copy2(gpxOrig, gpxfile)
 		else:
 			if (list_options["rcd_gpxfile"]==""):
-				logging.debug('removing gpxfile') # ein?
-				logging.debug('updating bbdd') #ein?
-		cells,values = self._formatRecord(list_options)
+				logging.debug('Activity not based in GPX file') # ein?
+		logging.debug('Updating bbdd')
+		cells,values = self._formatRecordNew(list_options)
 		self.pytrainer_main.ddbb.update("records",cells,values," id_record=%d" %int(id_record))
 		if equipment is not None:
 			self._update_record_equipment(id_record, equipment)
@@ -332,21 +342,25 @@ class Record:
 					"sports.name,date,distance,time,beats,comments,average,calories,id_record,title,upositive,unegative,maxspeed,maxpace,pace,maxbeats,date_time_utc,date_time_local",
 					"id_record=\"%s\" and records.sport=sports.id_sports" %id_record)
 
+	def format_date(self, date):
+		return date.strftime("%Y-%m-%d")
+
 	def getrecordList(self,date, id_sport=None):
 		logging.debug('--')
 		if not id_sport:
-			return self.pytrainer_main.ddbb.select("records,sports",
-					"sports.name,date,distance,time,beats,comments,average,calories,id_record,maxspeed,maxbeats,date_time_utc,date_time_local",
-					"date=\"%s\" and records.sport=sports.id_sports" %date)
+			# outer join on sport id to workaround bug where sport reference is null on records from GPX import
+			return self.pytrainer_main.ddbb.select("records left outer join sports on records.sport=sports.id_sports",
+					"sports.name,date,distance,time,beats,comments,average,calories,id_record,maxspeed,maxbeats,date_time_utc,date_time_local,upositive,unegative",
+					"date=\"%s\" " %self.format_date(date))
 		else:
 			return self.pytrainer_main.ddbb.select("records,sports",
-					"sports.name,date,distance,time,beats,comments,average,calories,id_record,maxspeed,maxbeats,date_time_utc,date_time_local",
-					"date=\"%s\" and sports.id_sports=\"%s\" and records.sport=sports.id_sports" %(date,id_sport))
+					"sports.name,date,distance,time,beats,comments,average,calories,id_record,maxspeed,maxbeats,date_time_utc,date_time_local,upositive,unegative",
+					"date=\"%s\" and sports.id_sports=\"%s\" and records.sport=sports.id_sports" %(self.format_date(date),id_sport))
 
 	def getLaps(self, id_record):
 		logging.debug('--')
 		laps = self.pytrainer_main.ddbb.select("laps",
-					"id_lap, record, elapsed_time, distance, start_lat, start_lon, end_lat, end_lon, calories, lap_number",
+					"id_lap, record, elapsed_time, distance, start_lat, start_lon, end_lat, end_lon, calories, lap_number, intensity, max_speed, avg_hr, max_hr, laptrigger, comments",
 					"record=\"%s\"" % id_record)
 		if laps is None or laps == []:  #No laps stored - update DB
 			logging.debug("No laps in DB for record %d" % id_record)
@@ -363,7 +377,7 @@ class Record:
 					self.insertLaps(lap_keys,lap.values())
 			#Try to get lap info again #TODO? refactor
 			laps = self.pytrainer_main.ddbb.select("laps",
-					"id_lap, record, elapsed_time, distance, start_lat, start_lon, end_lat, end_lon, calories, lap_number",
+					"id_lap, record, elapsed_time, distance, start_lat, start_lon, end_lat, end_lon, calories, lap_number, intensity, max_speed, avg_hr, max_hr, laptrigger, comments",
 					"record=\"%s\"" % id_record)
 		return laps
 
@@ -389,15 +403,17 @@ class Record:
 			record_equipment.append(equipment_item)
 		return record_equipment
 
-	def getrecordPeriod(self,date_ini, date_end, sport=None):
+	def getrecordPeriod(self, date_range, sport=None):
 		#TODO This is essentially the same as getrecordPeriodSport (except date ranges) - need to look at merging the two
+		date_ini = self.format_date(date_range.start_date)
+		date_end = self.format_date(date_range.end_date)
 		tables = "records,sports"
 		if not sport:
 			condition = "date>=\"%s\" and date<=\"%s\" and records.sport=sports.id_sports" %(date_ini,date_end)
 		else:
 			condition = "date>=\"%s\" and date<=\"%s\" and records.sport=sports.id_sports and sports.id_sports=\"%s\"" %(date_ini,date_end, sport)
 
-		return self.pytrainer_main.ddbb.select(tables,"date,distance,time,beats,comments,average,calories,maxspeed,maxbeats, sports.name", condition)
+		return self.pytrainer_main.ddbb.select(tables,"date,distance,time,beats,comments,average,calories,maxspeed,maxbeats, sports.name,upositive,unegative", condition)
 
 	def getrecordPeriodSport(self,date_ini, date_end,sport):
 		if not sport:
@@ -408,60 +424,64 @@ class Record:
 			condition = "date>\"%s\" and date<\"%s\" and records.sport=sports.id_sports and sports.id_sports=\"%s\"" %(date_ini,date_end,sport)
 
 		return self.pytrainer_main.ddbb.select(tables,
-					"date,distance,time,beats,comments,average,calories,maxspeed,maxbeats",
+					"date,distance,time,beats,comments,average,calories,maxspeed,maxbeats,upositive,unegative",
 					condition)
+		
+	def _get_sport(self, sport_name):
+		return self._sport_service.get_sport_by_name(sport_name)
 
-	def getSportMet(self,sport):
+	def getSportMet(self,sport_name):
+		"""Deprecated: use sport.met"""
 		logging.debug('--')
-		return self.pytrainer_main.ddbb.select("sports","met","name=\"%s\"" %(sport))[0][0]
+		return self._get_sport(sport_name).met
 
-	def getSportWeight(self,sport):
+	def getSportWeight(self,sport_name):
+		"""Deprecated: use sport.weight"""
 		logging.debug('--')
-		return self.pytrainer_main.ddbb.select("sports","weight","name=\"%s\"" %(sport))[0][0]
+		return self._get_sport(sport_name).weight
 
-	def getSportId(self,sport,add=None):
-		"""31.08.2008 - dgranda
-		Retrieves id_sports from provided sport. If add is not set to None, sport is added to the system
+	def getSportId(self, sport_name, add=None):
+		"""Deprecated: use sport_service.get_sport_by_name()
+		
+		Get the id of a sport by name, optionally adding a new sport if
+		none exists with the given name.
 		arguments:
-			sport: sport's name to get id from
-			add: attribute to add or not the sport to db
-		returns: id_sports from provided sport"""
-		logging.debug('>>')
-		sport_id=None
-		if sport == "" or sport is None:
-			return sport_id
-		try:
-			sport_id = self.pytrainer_main.ddbb.select("sports","id_sports","name=\"%s\"" %(sport))[0][0]
-		except:
-			logging.error('Error retrieving id_sports from '+ str(sport))
-			#traceback.print_last()
-			if add is None:
-				logging.debug('Sport '+str(sport)+' will not be added to DB')
-			else:
-				logging.debug('Adding sport '+str(sport)+' to DB')
-				sport_id = self.addNewSport(sport,"0","0")
-		logging.debug('<<')
-		return sport_id
+			sport_name: sport's name to get id for
+			add: whether the sport should be added if not found
+		returns: id for sport with given name or None"""
+		if sport_name is None:
+			return None
+		sport = self._get_sport(sport_name)
+		if sport is None:
+			logging.debug("No sport with name: '%s'", str(sport_name))
+			if add is not None:
+				logging.debug("Adding sport '%s'", str(sport_name))
+				new_sport = Sport()
+				new_sport.name = unicode(sport_name)
+				sport = self._sport_service.store_sport(new_sport)
+		return None if sport is None else sport.id
 
-	def addNewSport(self,sport,met,weight):
-		"""31.08.2008 - dgranda
-		Copied from Profile class, adds a new sport.
-		arguments:
-			sport: sport's name
-			met:
-			weight:
-		returns: id_sports from new sport"""
-		logging.debug(">>")
-		logging.debug("Adding new sport: "+sport+"|"+weight+"|"+met)
-		sportT = [sport,met,weight]
-		self.pytrainer_main.ddbb.insert("sports","name,met,weight",sportT)
-		sport_id = self.pytrainer_main.ddbb.select("sports","id_sports","name=\"%s\"" %(sport))[0][0]
-		logging.debug("<<")
-		return sport_id
+	def getLastRecordDateString(self, sport_id = None):
+		"""
+		Retrieve date (string format) of last record stored in DB. It may select per sport
+		"""
+		logging.debug("--")
+		#select date from records order by date desc limit 1;
+		query_cond = None
+		if sport_id is not None:
+			query_cond = "sport = %s" % sport_id
+		result = self.pytrainer_main.ddbb.select("records", "date", query_cond, "order by date desc limit 1")
+        # It returns a list of tuples, so we take first element of list (tuple) and then select first element
+		return result[0][0]
 
 	def getAllrecord(self):
+		"""
+		Retrieve all record data (no lap nor equipment) stored in database. Initially intended for csv export
+		arguments:
+		returns: list of data sorted by date desc"""
 		logging.debug('--')
-		return self.pytrainer_main.ddbb.select("records", "date,distance,time,beats,comments,average,calories")
+		return self.pytrainer_main.ddbb.select("records,sports", "date_time_local,title,sports.name,distance,duration,average,maxspeed,pace,maxpace,beats,maxbeats,calories,upositive,unegative,comments",
+            "sports.id_sports = records.sport","order by date_time_local asc")
 
 	def getAllRecordList(self):
 		logging.debug('--')
@@ -470,7 +490,7 @@ class Record:
 			"sports.id_sports = records.sport order by date desc")
 
 	def getRecordListByCondition(self,condition):
-		logging.debug('--')
+		logging.debug('>>')
 		if condition is None:
 			return self.getAllRecordList()
 		else:
@@ -481,13 +501,12 @@ class Record:
 
 	def getRecordDayList(self,date, id_sport=None):
 		logging.debug('>>')
-		year,month,day = date.split("-")
-		logging.debug('Retrieving data for '+year+'.'+month+'.'+day)
+		logging.debug('Retrieving data for ' + str(date))
 		# Why is looking for all days of the same month?
 		if not id_sport:
-			records = self.pytrainer_main.ddbb.select("records","date","date LIKE '"+year+"-"+month+"-%'")
+			records = self.pytrainer_main.ddbb.select("records","date","date LIKE '" +str(date.year)+"-"+date.strftime("%m")+"-%'")
 		else:
-			records = self.pytrainer_main.ddbb.select("records","date","date LIKE \"%s-%s-%%\" and sport=\"%s\"" %(year,month,id_sport))
+			records = self.pytrainer_main.ddbb.select("records","date","date LIKE \"%d-%0.2d-%%\" and sport=\"%s\"" %(date.year,date.month,id_sport))
 		logging.debug('Found '+str(len(records))+' entries')
 		day_list = []
 		for i in records:
@@ -510,7 +529,7 @@ class Record:
 			logging.debug('Found '+str(len(tracks))+' tracks')
 			self._select_trkfromgpx(gpxfile,tracks)
 		else:
-			msg = _("pyTrainer can't import data from your gpx file")
+			msg = _("pytrainer can't import data from your gpx file")
 			from gui.warning import Warning
 			warning = Warning(self.data_path)
 			warning.set_text(msg)
@@ -524,8 +543,10 @@ class Record:
 		heartrate = gpx.getHeartRateAverage()
 		date = gpx.getDate()
 		calories = gpx.getCalories()
+		start_time = gpx.getStart_time()
 
 		self.recordwindow.rcd_date.set_text(date)
+		self.recordwindow.rcd_starttime.set_text(start_time)
 		self.recordwindow.rcd_upositive.set_text(str(upositive))
 		self.recordwindow.rcd_unegative.set_text(str(unegative))
 		self.recordwindow.rcd_beats.set_text(str(heartrate))
@@ -534,8 +555,7 @@ class Record:
 		self.recordwindow.set_maxspeed(maxspeed)
 		self.recordwindow.set_maxhr(maxheartrate)
 		self.recordwindow.set_recordtime(time/60.0/60.0)
-		self.recordwindow.on_calcaverage_clicked(None)
-		self.recordwindow.on_calcpace_clicked(None)
+		self.recordwindow.on_calcavs_clicked(None)
 		self.recordwindow.on_calccalories_clicked(None)
 		self.recordwindow.rcd_maxpace.set_text("%d.%02d" %((3600/maxspeed)/60,(3600/maxspeed)%60))
 		logging.debug('<<')
@@ -554,18 +574,6 @@ class Record:
 		selectrckdialog.run()
 		logging.debug('<<')
 
-	def newGpxRecord(self,gpxfile,list_sport): #TODO Not used?
-		logging.debug('>>')
-		logging.debug("opening a new window record "+self.data_path+'|'+gpxfile+'|'+str(list_sport))
-		self.recordwindow = WindowRecord(self._equipment_service, self.data_path, list_sport,self, None)
-		logging.debug('setting text in window '+ gpxfile)
-		self.recordwindow.rcd_gpxfile.set_text(gpxfile)
-		logging.debug('retrieving data from gpx file')
-		self.actualize_fromgpx(gpxfile)
-		logging.debug('Launching window...')
-		self.recordwindow.run()
-		logging.debug('<<')
-
 	def importFromGPX(self, gpxFile, sport):
 		"""
 		Add a record from a valid pytrainer type GPX file
