@@ -17,6 +17,7 @@
 #Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 import logging
+from pytrainer.lib.date import unixtime2date
 
 class Waypoint:
 	def __init__(self, data_path = None, parent = None):
@@ -80,7 +81,7 @@ class Waypoint:
 		tracks = gpx.getTrackRoutes()
 
 		if len(tracks) > 1:
-			time = self.date.unixtime2date(tracks[0][1])
+			time = unixtime2date(tracks[0][1])
 			self.recordwindow.rcd_date.set_text(time)
 			self._actualize_fromgpx(gpx)
 		else:
@@ -100,5 +101,5 @@ class Waypoint:
 		self.recordwindow.rcd_unegative.set_text(str(unegative))
 		self.recordwindow.set_distance(distance)
 		self.recordwindow.set_recordtime(time/60.0/60.0)
-		self.recordwindow.on_calcaverage_clicked(None)
+		self.recordwindow.on_calcavs_clicked(None)
 		logging.debug("<<")
